# coding: utf-8

from __future__ import unicode_literals

import grp
import json
import os
import pwd
import subprocess
import logging

from collections import Iterable

from tionix_autosetup.helpers import installed_tionix_modules

logger = logging.getLogger(__name__)

RENAMING_POSTFIX = '.d'
EXAMPLE_POSTFIX = '.example'
MODULES_CONF_DIR = u'/etc/tionix'
HIDDEN_PASSWORD = "********"


class AutosetupException(Exception):
    pass


class ConnectionCheckException(AutosetupException):
    pass


def fabric(fn):
    def wrapper(*args, **kwargs):
        product = fn(*args, **kwargs)
        product.collect()
        return product

    return wrapper


class ExecutorResult(object):
    def __init__(self, pre_check=None, post_check=None, run_result=None,
                 run_check=None):
        self.pre_check = pre_check
        self.post_check = post_check
        self.run_result = run_result
        self.run_check = run_check

    def failed(self):
        return self.post_check is False or self.run_check is False


class Executor(object):
    commands = ()
    variables_dict = None
    force = False
    result_log_level = logging.DEBUG
    critical_operation = False

    def __init__(self, variable_box=None, **kwargs):
        super(Executor, self).__init__()
        self.result = ExecutorResult()
        self.extra_kwargs = getattr(variable_box, 'kwargs', {})
        self.force = self.extra_kwargs.get('force', False)
        self.variables_dict = (variable_box.to_dict().copy()
                               if variable_box else {})

    # pylint: disable=no-member
    @staticmethod
    def log_message(cmd, safe_cmd, log_level, msg):
        if logger.isEnabledFor(logging.DEBUG):

            if cmd and safe_cmd and safe_cmd != cmd:
                safe_fmt_msg = msg.format(cmd=safe_cmd)
                fmt_msg = msg.format(cmd=cmd)
                logger.log(log_level, fmt_msg,
                           extra={'safe_msg': safe_fmt_msg})

            elif safe_cmd == cmd or (cmd and not safe_cmd):
                logger.log(log_level, msg.format(cmd=cmd), )
        else:
            if cmd and safe_cmd and safe_cmd != cmd:
                logger.log(log_level, msg.format(cmd=safe_cmd))

            elif safe_cmd == cmd or (cmd and not safe_cmd):
                logger.log(log_level, msg.format(cmd=cmd))

    @classmethod
    def sh_exec(cls, cmd, safe_cmd=None, log_msg=True):
        if log_msg:
            msg = 'Executing command: {cmd}'
            cls.log_message(cmd, safe_cmd, logging.INFO, msg)
        try:
            p = subprocess.Popen(cmd, shell=True,
                                 stdin=subprocess.PIPE,
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.STDOUT)
            result = p.communicate()[0]
        except OSError as exc:
            if log_msg:
                msg = 'Command failed: {cmd} with error ' + unicode(exc) + ':'
                cls.log_message(cmd, safe_cmd, logging.CRITICAL, msg)
            return
        else:
            if result and log_msg:
                msg = result.decode('utf8')
                logger.log(
                    cls.result_log_level, 'The result is: {0}'.format(msg))
        return result

    @staticmethod
    def _passwords_escape(var_dict):
        unsafe_chunk = 'pass'
        safe_var_dict = var_dict.copy()
        unsafe_vars = {key: HIDDEN_PASSWORD for key, value in var_dict.items()
                       if unsafe_chunk in key.lower()}
        safe_var_dict.update(unsafe_vars)
        return safe_var_dict

    def pre_check(self, *arg, **kwargs):
        """Perform check before running commands. If fails, commands won't run
        """
        return True

    def post_check(self, *arg, **kwargs):
        """Perform check after running commands.
        """
        return True

    def run(self, **kwargs):
        self.result.pre_check = False
        self.result.run_check = True
        if self.pre_check(**kwargs):
            self.result.pre_check = True
            try:
                self.result.run_result = self._run(**kwargs)
            except Exception as err:
                self.result.run_check = False
                self.result.run_result = unicode(err)
                logger.error("Run error: {0}".format(unicode(err)))
            self.result.post_check = self.post_check(**kwargs)
        return ExecutorResult(
            pre_check=self.result.pre_check,
            post_check=self.result.post_check,
            run_result=self.result.run_result,
            run_check=self.result.run_check,
        )

    def _run(self, **kwargs):
        var_dict = self.variables_dict
        var_dict.update(kwargs)
        safe_var_dict = self._passwords_escape(var_dict)

        results = []
        for r_cmd in self.commands:
            if isinstance(r_cmd, tuple):
                r_cmd = ' || '.join(r_cmd)
            cmd = r_cmd.format(**var_dict)
            safe_cmd = safe_var_dict and r_cmd.format(**safe_var_dict)
            results.append(self.sh_exec(cmd=cmd, safe_cmd=safe_cmd))
        return results


class BaseConnectionCheckExecutor(Executor):
    def check_connection(self, var_dict):
        raise NotImplementedError

    def _run(self, **kwargs):
        var_dict = self.variables_dict
        var_dict.update(kwargs)
        return self.check_connection(var_dict)


class AllModulesRemovedMixin(Executor):
    def pre_check(self, *arg, **kwargs):
        return not installed_tionix_modules()


class PolicyScopeExecutor(Executor):

    _os_params = (
        ' --os-password {AUTH_PASSWORD} '
        '--os-user-domain-name {USER_DOMAIN_NAME} '
        '--os-project-domain-name {PROJECT_DOMAIN_NAME} '
        '--os-auth-url {AUTH_URL} '
        '--os-username {AUTH_USERNAME} '
        '--os-identity-api-version {IDENTITY_API_VERSION} '
    )

    _os_scope_params = [
        '--os-project-name {PROJECT_NAME} --os-domain-name=""',
        '--os-domain-name {USER_DOMAIN_NAME} --os-project-name=""',
    ]

    def execute_command(self, cmd, log_msg=False, json_return=True):
        json_result = result = None
        var_dict = self.variables_dict
        safe_var_dict = self._passwords_escape(var_dict)
        cmd += self._os_params

        for scope_param in self._os_scope_params[:]:
            _cmd = cmd + scope_param
            formatted_cmd = _cmd.format(**var_dict)
            formatted_safe_cmd = safe_var_dict and _cmd.format(**safe_var_dict)
            result = self.sh_exec(formatted_cmd, safe_cmd=formatted_safe_cmd,
                                  log_msg=log_msg)
            try:
                json_result = json.loads(result)
            except ValueError:
                if (isinstance(result, basestring) and
                        'HTTP 403' in result.upper()):
                    continue

            if self._os_scope_params[0] != scope_param:
                self._os_scope_params.remove(scope_param)
                self._os_scope_params.insert(0, scope_param)
            break

        return json_result if json_return else result

    def _run(self, **kwargs):
        results = []
        for r_cmd in self.commands:
            if isinstance(r_cmd, tuple):
                r_cmd = ' || '.join(r_cmd)
            result = self.execute_command(r_cmd, log_msg=True,
                                          json_return=False)
            results.append(result)
        return results


class AdminRoleCheckMixin(PolicyScopeExecutor):

    @staticmethod
    def is_domain_in_roles(roles, domain_id):
        result = False
        if isinstance(roles, Iterable):
            for role in roles:
                if isinstance(role, dict):
                    if role.get('Domain') == domain_id:
                        result = True
                        break
        return result

    def check_domain_admin_role(self):
        exist = False
        osc_cmd = 'openstack role assignment list --user {DOMAIN_USER} -f json'
        admin_domain_id = self.variables_dict.get('ADMIN_DOMAIN_ID')

        if admin_domain_id:
            cmd_result = self.execute_command(osc_cmd, log_msg=True)
            if cmd_result is None:
                raise AutosetupException('Could not connect to Keystone '
                                         'to check admin role existence.')
            if self.is_domain_in_roles(cmd_result, admin_domain_id):
                exist = True

        return exist


class CloudDomainPolicyMixin(object):

    @property
    def cloud_domain_policy(self):
        kwargs = getattr(self, 'extra_kwargs', {})
        return kwargs.get('cloud_domain_policy', False)

    def pre_check(self, *arg, **kwargs):
        return self.cloud_domain_policy


class AppendPath(Executor):
    path = '/usr/sbin'

    def pre_check(self, *arg, **kwargs):
        return self.path not in os.environ['PATH']

    def _run(self, **kwargs):
        os.environ['PATH'] += ':'.join([os.environ['PATH'], self.path])


class Dirs(Executor):
    commands = (
        'mkdir {dir_name}',
        'chown -R {TNX_USERNAME}:{TNX_USERNAME} {dir_name}'
    )
    dirs = ()

    def run(self, **kwargs):
        result = []
        if not self.dirs:
            return result
        for dir_name in self.dirs:
            result.append(super(Dirs, self).run(**{'dir_name': dir_name}))
        return result

    @staticmethod
    def _is_owner(username, dirname):
        """ Check if the user is owner of dir """
        dir_stat = os.stat(dirname)
        owner_uid = dir_stat.st_uid
        owner_gid = dir_stat.st_gid
        try:
            uid = pwd.getpwnam(username).pw_uid
            gid = grp.getgrnam(username).gr_gid
        except KeyError:
            logger.critical("User doesn't exist: {0}".format(
                username))
            return False
        return (owner_uid == uid) and (owner_gid == gid)

    def pre_check(self, *arg, **kwargs):
        """ Check if dir exists and user is owner before run

            Check passed if user doesn't exist or user is not owner
        """
        dir_name = kwargs.get('dir_name')
        user = self.variables_dict.get('TNX_USERNAME')
        if os.path.exists(dir_name) and self._is_owner(user, dir_name):
            logger.info('Dir {0} already exists and user {1} is owner'.format(
                dir_name, user))
            return False
        return True

    def post_check(self, *arg, **kwargs):
        """ Check if dir exists and user is owner after run

            Check passed if both conditions are true
        """
        dir_name = kwargs.get('dir_name')
        user = self.variables_dict.get('TNX_USERNAME')
        if not os.path.exists(dir_name):
            logger.critical("Folder wasn't created: {0}".format(dir_name))
            return False
        if not self._is_owner(user, dir_name):
            logger.critical('User {0} is not owner of the folder: {1}'.format(
                user, dir_name))
            return False
        return True


class ModuleCheckMixin(object):

    def is_packages_installed(self, packages):
        result = True
        for package in packages:
            if not self.is_package_installed(package):
                logger.error('Package is not installed: {0}'.format(package))
                result = False
        return result


class ModuleInstallCheckMixin(ModuleCheckMixin):
    def post_check(self, *arg, **kwargs):
        """ Check if package installed after run """
        result = super(ModuleInstallCheckMixin, self).post_check(*arg,
                                                                 **kwargs)
        result = result and self.is_packages_installed(self.packages)
        return result


class ModuleUpdateCheckMixin(ModuleCheckMixin):
    def pre_check(self, *arg, **kwargs):
        """ Check if package installed before run """
        result = super(ModuleUpdateCheckMixin, self).pre_check(*arg, **kwargs)
        result = result and self.is_packages_installed(self.packages)
        return result


class ModuleUninstallCheckMixin(ModuleCheckMixin):

    def pre_check(self, *arg, **kwargs):
        """ Check if package installed before run """
        result = super(ModuleUninstallCheckMixin, self).pre_check(*arg,
                                                                  **kwargs)
        result = result and self.is_packages_installed(self.packages)
        return result

    def post_check(self, *arg, **kwargs):
        """ Check if package uninstalled after run """
        result = super(ModuleUninstallCheckMixin, self).post_check(*arg,
                                                                   **kwargs)
        if result:
            for package in self.packages:
                if self.is_package_installed(package):
                    logger.error("Couldn't uninstall package: {0}"
                                 "".format(package))
                    result = False
        return result
