# coding: utf-8

from __future__ import unicode_literals

import ConfigParser
import logging
import os
import re
import shutil

from datetime import datetime
import yaml
from jinja2 import FileSystemLoader
from jinja2.environment import Environment

from tionix_autosetup.executors.base import (
    Executor, MODULES_CONF_DIR, RENAMING_POSTFIX, EXAMPLE_POSTFIX)
from tionix_autosetup.helpers import remove_paths, fill_missing_keys
from tionix_autosetup.variables import (
    get_yaml_variables_matching, MODULE_PREFIX)


logger = logging.getLogger(__name__)

DEFAULT_SUPERVISOR_DIR = u'/etc/supervisor'

SUPERVISOR_CONF_DIR = DEFAULT_SUPERVISOR_DIR + u'/conf.d'


def read_config_file(file_path, config_parser=None):
    """Reads data from config to parser
    :param file_path: path to config file (eg. /etc/nova/nova.conf)
    :param config_parser: config parser instance
    """
    cfg = config_parser
    if not cfg:
        cfg = ConfigParser.ConfigParser()

    with open(file_path) as fh:
        cfg.readfp(fh)
    return cfg


def write_config_file(mode, output_line, config_path):
    base_dir = os.path.dirname(config_path)
    if not os.path.exists(base_dir):
        os.mkdir(base_dir)
    with open(config_path, mode) as fh:
        fh.writelines(output_line)


class Config(Executor):
    config_path = None
    template = None
    template_name = ''
    current_dir_path = os.path.dirname(os.path.abspath(__file__))
    templates_dir_path = os.path.join(current_dir_path, '../templates')
    rewrite = False

    def _run(self, **kwargs):
        result = []
        mode = 'w'

        if not self.rewrite and os.path.exists(self.config_path):
            logger.warning('Skipping file create for: {0}'.format(
                self.config_path))
        else:
            var_dict = self.variables_dict
            var_dict.update(kwargs)
            env = Environment(loader=FileSystemLoader(self.templates_dir_path))
            template = env.get_template(self.template_name)
            output_line = template.render(var_dict)
            write_config_file(mode, output_line, self.config_path)

        return result

    def post_check(self, *arg, **kwargs):
        """ Check if config exists after run

        Check passed if config_path exists
        """
        if not os.path.exists(self.config_path):
            logger.critical("Config file wasn't created: {0}".format(
                self.config_path))


class PathCheckMixin(object):
    fail_message = 'Failed to delete following paths:\n{0}'
    success_message = None

    def post_check(self, *arg, **kwargs):
        failed_paths = []
        paths = getattr(self, 'paths')
        for path in paths:
            if os.path.exists(path):
                failed_paths.append(path)

        if failed_paths:
            logger.error(self.fail_message.format('\n'.join(failed_paths)))
        elif self.success_message:
            logger.debug(self.success_message)


class DropPath(PathCheckMixin, Executor):
    paths = ()

    def pre_check(self, *arg, **kwargs):
        return self.force

    def _run(self, **kwargs):
        remove_paths(self.paths)


class DropConfig(DropPath):
    config_name = None
    config_dir = MODULES_CONF_DIR
    renaming_postfix = RENAMING_POSTFIX

    @property
    def paths(self):
        return [os.path.join(self.config_dir, self.config_name)]

    def pre_check(self, *arg, **kwargs):
        return self.force

    def _run(self, **kwargs):
        super(DropConfig, self)._run(**kwargs)
        renamed_paths = [path + self.renaming_postfix for path in self.paths]
        remove_paths(renamed_paths)

    @staticmethod
    def search_files(path_dir, prefix):
        if os.path.exists(path_dir):
            return (os.path.join(path_dir, f) for f in os.listdir(path_dir)
                    if f.startswith(prefix))
        return ()


class RenameConfig(PathCheckMixin, Executor):
    config_name = None
    config_dir = MODULES_CONF_DIR
    renaming_postfix = RENAMING_POSTFIX
    fail_message = 'Failed to rename following files:\n{0}'

    @property
    def paths(self):
        return [os.path.join(self.config_dir, self.config_name)]

    def _run(self, **kwargs):
        for path in self.paths:
            if os.path.isfile(path):
                os.rename(path, path + self.renaming_postfix)


class ActualizeConfig(Executor):
    """
    Compares config.yaml.example and config.yaml and adds missing keys
     from config.yaml.example to config.yaml.
    """
    config_name = None
    config_dir = MODULES_CONF_DIR

    @property
    def yaml_file_path(self):
        return os.path.join(self.config_dir, self.config_name)

    @property
    def yaml_example_file_path(self):
        return os.path.join(
            self.config_dir, self.config_name + EXAMPLE_POSTFIX)

    def pre_check(self, *arg, **kwargs):
        return (os.path.exists(self.yaml_file_path) and
                os.path.exists(self.yaml_example_file_path))

    def _run(self, **kwargs):
        with open(self.yaml_file_path, mode='r') as yaml_file:
            yaml_orig = yaml.load(yaml_file)
        with open(self.yaml_example_file_path, mode='r') as yaml_file_example:
            yaml_example = yaml.load(yaml_file_example)

        if yaml_orig != yaml_example:
            yaml_updated = fill_missing_keys(yaml_orig, etalon=yaml_example)
            if yaml_updated != yaml_orig:
                with open(self.yaml_file_path, mode='w') as yaml_file:
                    yaml.dump(yaml_updated, yaml_file,
                              default_flow_style=False)


class DropSupervisorConfig(DropConfig):
    module_config_prefix = None
    config_dir = SUPERVISOR_CONF_DIR

    @property
    def paths(self):
        return self.search_files(
            self.config_dir,
            self.module_config_prefix
        )


class ConfigFileParser(Executor):
    """
    Class for process any config files
    """
    config_path = None
    delete = False

    def __init__(self, variable_box=None):
        super(ConfigFileParser, self).__init__(variable_box)
        self.new_config_data = {}
        self.data_for_process = {}
        self.setup_data_for_process()

    def setup_new_config_data(self):
        pass

    def setup_data_for_process(self):
        data = self.read_config_file()

        self.setup_new_config_data()

        if data is not None:
            for config_string, config_pattern in self.new_config_data.items():
                update = False
                pattern_found = config_pattern.findall(data)

                if self.delete:
                    if self.force and pattern_found:
                        formatted_str = self.get_formatted_str(config_string)
                        self.data_for_process[formatted_str] = (config_pattern,
                                                                update)
                else:
                    if pattern_found:
                        update = True

                    formatted_str = self.get_formatted_str(config_string)
                    if formatted_str not in data:
                        self.data_for_process[formatted_str] = (config_pattern,
                                                                update)

    def make_pattern(self, cfg):
        cfg = self.get_formatted_str(cfg)
        return re.compile(re.escape(cfg))

    def get_formatted_str(self, config_string):
        return config_string.format(**self.variables_dict)

    def read_config_file(self):
        data = None
        if self.is_config_exist():
            with open(self.config_path) as fh:
                data = fh.read()
        return data

    def is_config_exist(self):
        result = True
        if self.config_path is None:
            result = False
        elif not os.path.isfile(self.config_path):
            logger.warning("File {0} doesn't exist".format(self.config_path))
            result = False
        elif not os.access(self.config_path, os.W_OK):
            logger.warning(
                "Access denied to file {0}".format(self.config_path))
            result = False
        return result

    def pre_check(self, *arg, **kwargs):
        result = False
        data = self.read_config_file()

        if data is not None and self.data_for_process:
            result = True

        return result

    def _run(self, **kwargs):
        data = self.read_config_file()

        if data is not None:
            for cfg_str, (pattern, update) in self.data_for_process.items():
                if update:
                    data = re.sub(pattern, cfg_str, data)
                elif self.delete:
                    data = re.sub(pattern, '', data)
                else:
                    data += cfg_str

            with open(self.config_path, 'w') as fh:
                fh.write(data)
        return

    def post_check(self, *arg, **kwargs):
        result = True
        data = self.read_config_file()

        if data is None:
            result = False
        elif not self.data_for_process:
            pass
        elif self.delete:
            if any(pattern.findall(data) for cfg_str, (pattern, update)
                   in self.data_for_process.items()):
                logger.error(
                    "Some module's config wasn't removed from "
                    "file {0}".format(self.config_path))
                result = False

        elif any(cfg_str not in data for cfg_str, _tail
                 in self.data_for_process.items()):
            logger.error("File {0} wasn't updated".format(
                self.config_path))
            result = False

        return result


# todo Add tuple of config paths support
class ConfigWithSectionsParser(Executor):
    """
    Class for process config files with sections (like win ini-files)
    Example:

        [Section1]
        var1=value_1
        var2=value_2

        [Section2]
        var1=value_1
        var2=value_2
    """

    config_path = None
    config_section = None
    config_keys = None
    __config = None
    append_to_section_value = False

    @property
    def config(self):
        if not self.__config:
            self.__config = ConfigParser.ConfigParser()
        return self.__config

    def is_config_correct(self):
        """ Check if config file variables have correct values """
        result = True

        read_config_file(self.config_path, config_parser=self.config)

        for key, value in self.config_keys.items():
            val = value.format(**self.variables_dict)
            conf_val = None
            try:
                conf_val = self.config.get(self.config_section, key)
            except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
                result = False

            if val != conf_val:
                result = False
            if result:
                break

        return result

    def pre_check(self, *arg, **kwargs):
        if not os.path.exists(self.config_path):
            logger.error("File {0} doesn't exist".format(self.config_path))
            return False
        return not self.is_config_correct()

    def _run(self, **kwargs):
        for key, value in self.config_keys.items():
            if not (self.config.has_section(self.config_section) or
                    self.config_section.lower() == "default"):
                self.config.add_section(self.config_section)

            value = value.format(**self.variables_dict)

            if self.append_to_section_value:
                try:
                    conf_val = self.config.get(self.config_section, key)
                except (ConfigParser.NoOptionError,
                        ConfigParser.NoSectionError):
                    self.config.set(self.config_section, key, value)
                else:
                    if value not in conf_val:
                        value = '{0} {1}'.format(conf_val, value)
                        self.config.set(self.config_section, key, value)
            else:
                self.config.set(self.config_section, key, value)

        with open(self.config_path, 'w') as fh:
            self.config.write(fh)

    def post_check(self, *arg, **kwargs):
        return self.is_config_correct()


class UpdateSudoers(Executor):
    """ Add user to /etc/sudoers """

    config_template = "\n{TNX_USERNAME} ALL=(ALL) NOPASSWD: ALL"
    file_path = '/etc/sudoers'
    commands = (
        'echo "{0}" >> {1}'.format(config_template, file_path),
    )

    @property
    def config_str(self):
        return self.config_template.format(**self.variables_dict)

    def pre_check(self, *arg, **kwargs):
        """ Check if user is already in /etc/sudoers """
        result = True
        try:
            with open(self.file_path) as fh:
                data = fh.read()
        except IOError:
            logger.warning(
                "File {0} doesn't exist or "
                "you don't have permission".format(self.file_path))
            result = False
        else:
            if self.config_str in data:
                result = False
        return result

    def post_check(self, *arg, **kwargs):
        """ Check if user is in /etc/sudoers """
        result = True
        with open(self.file_path) as fh:
            data = fh.read()
        if self.config_str not in data:
            logger.error("Couldn't add user to sudoers list")
            result = False
        return result


class UpdateYamlConfig(Config):

    overwrite_configs = False
    rename_date_format = '%d%m%Y_%H%M%S'
    module_varables_names = ()
    module_config = True

    def __init__(self, variable_box=None, **kwargs):
        super(UpdateYamlConfig, self).__init__(variable_box, **kwargs)

        if hasattr(variable_box, 'kwargs'):
            self.overwrite_configs = variable_box.kwargs.get(
                'overwrite_configs', False)
            self.yaml_variables_matching = get_yaml_variables_matching()

    def pre_check(self, *arg, **kwargs):
        return self.overwrite_configs

    def _backup_cfg(self):
        """
        Backup existing yaml config in new file with renaming
        prefix as current datetime.
        """
        renaming_postfix = '.' + datetime.now().strftime(
            self.rename_date_format)
        backup_cfg_path = self.config_path + renaming_postfix
        shutil.copy(self.config_path, backup_cfg_path)

    def _find_keys_in_mappings(self, var_name, result=None, dict_obj=None,
                               parent_key=None):
        """
        Find keys of dict which represents path to target value.

        :param var_name: name of variable with new value
        :param result: list of dict keys
        :param dict_obj: dict with yaml keys and variable names mapping
        :param parent_key: parent key (for nested dict)

        For example:
        {
            'LOG_LEVEL': 'LOG_LEVEL',
            'NEUTRON_VERSION': 'NETWORK_API_VERSION',
            'CINDER_VERSION': 'VOLUME_API_VERSION',
            'KEYSTONE': {
                'auth_version': 'IDENTITY_API_VERSION',
                'compute_service_name': 'COMPUTE_SERVICE_NAME',
                'network_service_name': 'NETWORK_SERVICE_NAME',
            },
        }

        For variable with name COMPUTE_SERVICE_NAME result will
        ['KEYSTONE', 'compute_service_name']
        """
        found = False
        result = result or []
        dict_obj = dict_obj or self.yaml_variables_matching
        for key, value in dict_obj.items():
            if isinstance(value, dict):
                result.append(key)
                result, found = self._find_keys_in_mappings(
                    var_name, result, value, key)
                if found:
                    break
            else:
                if value == var_name:
                    result.append(key)
                    found = True
                    break
        else:
            if parent_key in result:
                result.remove(parent_key)
        return result, found

    @staticmethod
    def _update_dict_value(keys, content, new_value):
        """
        Update dict using list of keys

        :param keys: dict keys
        :param content: dict to update
        :param new_value: new value
        """
        _content = None
        last_item = keys.pop()

        for key in keys:
            _content = _content if _content is not None else content
            if key not in _content:
                _content[key] = {}
            _content = _content.get(key)

        if _content is None:
            content[last_item] = new_value
        else:
            _content[last_item] = new_value

    def _update_yaml_by_variables(self, **kwargs):
        """
        Open existing yaml config file and load content.
        Find keys of dict which represents path to target value.
        Using keys update value. Write updated dict to yaml file.
        """
        try:
            with open(self.config_path) as file_handler:
                yaml_dict = yaml.load(file_handler)
        except EnvironmentError:
            pass
        else:
            var_dict = self.variables_dict
            var_dict.update(kwargs)

            for var_name, value in var_dict.items():
                if ((MODULE_PREFIX in var_name and self.module_config) or
                        (MODULE_PREFIX not in var_name and
                         not self.module_config) or
                        (var_name in self.module_varables_names)):

                    var_name = var_name.replace(MODULE_PREFIX, '')
                    keys, found = self._find_keys_in_mappings(var_name)
                    if found:
                        self._update_dict_value(keys, yaml_dict, value)

            with open(self.config_path, mode='w') as yaml_file:
                yaml.safe_dump(yaml_dict, yaml_file,
                               default_flow_style=False, indent=4)

    def _run(self, **kwargs):
        self._backup_cfg()
        self._update_yaml_by_variables(**kwargs)
