# coding: utf-8

"""
Helpers methods
"""

import argparse
import collections
import logging
import os

from datetime import datetime
from platform import linux_distribution

import shutil
import sys
from StringIO import StringIO
from contextlib import contextmanager
import subprocess
import pkg_resources

from pkg_resources import (DistributionNotFound, VersionConflict,
                           ContextualVersionConflict)


from tionix_autosetup import BASE_DIR

logger = logging.getLogger(__name__)
LICENSE_CONTROL_GROUP = u'tionix_license_control'
AUTOSETUP_CONF_DIR = u'/etc/tionix/autosetup/'
BEFORE_AUTOSETUP_ENV = AUTOSETUP_CONF_DIR + u'pip_packages_before_{0}.env'
ACTUAL_AUTOSETUP_ENV = AUTOSETUP_CONF_DIR + u'pip_packages_actual.env'
PYPI_LIST_FILE = AUTOSETUP_CONF_DIR + u'pypi_list.txt'


class classproperty(property):
    def __get__(self, instance, cls=None):
        getter = getattr(self, 'fget')
        return getter(cls)


def get_package_name(base_dir=None):
    base_dir = base_dir or BASE_DIR
    dirs = reversed(os.path.abspath(base_dir).split('/'))
    try:
        package_name = [dir_name.replace('_', '-') for dir_name in dirs
                        if 'tionix_' in dir_name][0]
    except IndexError:
        package_name = None
    return package_name


def get_version():
    """ Get installed package version """
    version = None
    package_name = get_package_name()
    try:
        version = pkg_resources.get_distribution(package_name).version
    except DistributionNotFound:
        logger.error(u'Package not found: {0}'.format(package_name))
    except (VersionConflict, ContextualVersionConflict) as err:
        logger.warning(u'Version conflict found: {0}'.format(err))
    return version


def get_formatted_version():
    return u"{0} {1}".format(
        get_package_name(), get_version()
    )


def create_argparser(*args, **kwargs):
    parser = argparse.ArgumentParser(*args, **kwargs)
    parser.add_argument(
        '--version', action='version', version=get_formatted_version())
    return parser


class OperationsEnum(object):
    INSTALL = u'install'
    UPDATE = u'update'
    UNINSTALL = u'uninstall'
    CHECK_CONNECTION = u'connections_check'
    POST_CHECK = u'post_check'
    ROLLBACK = u'rollback'

    values = (INSTALL, UPDATE, UNINSTALL, CHECK_CONNECTION, POST_CHECK,
              ROLLBACK)


def get_licensed_module_names(group=LICENSE_CONTROL_GROUP, module_names=None):
    if not isinstance(module_names, collections.Iterable):
        module_names = [module_names]
    entry_points = []
    for module_name in module_names:
        entry_points.extend(
            pkg_resources.iter_entry_points(group=group,
                                            name=module_name)
        )
    return [entry_point.name for entry_point in entry_points]


def remove_paths(paths):
    for path in paths:
        if os.path.isfile(path):
            os.remove(path)
        else:
            shutil.rmtree(path, ignore_errors=True)


def check_module_installed(package_name):
    try:
        pkg_resources.get_distribution(package_name)
    except pkg_resources.DistributionNotFound:
        return False
    except (pkg_resources.VersionConflict,
            pkg_resources.ContextualVersionConflict):
        pass
    return True


class Versions(object):
    MITAKA = '13'
    LIBERTY = '12'


def get_package_version(distr):
    try:
        package = pkg_resources.get_distribution(distr)
    except (TypeError, pkg_resources.DistributionNotFound):
        return None
    else:
        return package.version


def get_formatting_distr(distr):
    version = get_package_version(distr)
    return version.split('.')[0] if version else ''


def installed_tionix_modules():
    from tionix_autosetup.packages import PACKAGES
    return [module_name for module_name, pip_name in PACKAGES.pip_names.items()
            if check_module_installed(pip_name)]


@contextmanager
def suppress_stdout():
    new_stdout = StringIO()
    old_stdout = sys.stdout
    old_stderr = sys.stderr
    logging.disable(logging.ERROR)
    try:
        sys.stdout = new_stdout
        sys.stderr = new_stdout
        yield
    finally:
        sys.stdout = old_stdout
        sys.stderr = old_stderr
        logging.disable(logging.NOTSET)


class Platform(object):
    ALT_LINUX = u'alt linux'
    UBUNTU = u'ubuntu'
    DEFAULT = UBUNTU

    DISTRIBUTIONS = (ALT_LINUX, UBUNTU)

    _distribution = None

    @classproperty
    def distribution(cls):  # pylint:disable=no-self-argument
        if cls._distribution is None:
            distribution, _version, _id = linux_distribution()

            if distribution is not None:
                cls._distribution = unicode(distribution).lower().strip()

            if cls._distribution not in cls.DISTRIBUTIONS:
                logger.warning(
                    u'Current distribution is not supported: {0}. '
                    u'Distribution is to default: {1}'.format(
                        distribution, cls.DEFAULT))
                cls._distribution = cls.DEFAULT

        return cls._distribution


def save_installed_packages_info(file_path=ACTUAL_AUTOSETUP_ENV):
    commands = (
        u'mkdir -p {0}'.format(AUTOSETUP_CONF_DIR),
        u'pip freeze > {0}'.format(file_path),
    )
    for cmd in commands:
        try:
            popen = subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE,
                                     stdout=subprocess.PIPE,
                                     stderr=subprocess.STDOUT)
            _stdout, stderr = popen.communicate()
            if stderr:
                logger.warning(u'Error in subprocess: {0}'.format(stderr))
        except OSError as err:
            logger.warning(u'Command failed: {cmd} '
                           u'with error {err}.'.format(cmd=cmd, err=err))

    logger.debug(u'Installed packages info successfully saved '
                 u'to file: {0}'.format(file_path))

    return file_path


def save_used_pypi_host(variable_box, file_path=PYPI_LIST_FILE):
    variable_box = variable_box.to_dict()
    pypi_host = variable_box.get('PYPI_HOST')
    if pypi_host:
        date_time = datetime.now().strftime("%Y.%m.%d-%H:%M:%S")
        pypi_address = '{0};{1}'.format(date_time, pypi_host)
        try:
            if not os.path.exists(os.path.dirname(file_path)):
                os.makedirs(file_path)
            with open(file_path, 'a+') as f:
                f.write(pypi_address + '\n')
        except (IOError, OSError) as err:
            logger.error(u'Unable to save PYPI_HOST to file: {0}'.format(err))


def fill_missing_keys(original, etalon):
    """
    Compares etalon and original dicts and recursively fills missing keys.

    :param original: dict
    :param etalon: dict
    :return: dict with all keys from etalon
    """
    result_dict = original.copy()

    for key, value in etalon.items():
        if key not in result_dict:
            result_dict[key] = value
        elif isinstance(value, dict):
            result_dict[key] = fill_missing_keys(result_dict[key], value)

    return result_dict


class PackageTypesEnum(object):
    PIP = u'pip'
    RPM = u'rpm'
    values = (PIP, RPM)


class ManagerTypesEnum(object):
    PIP = 'pip'
    APT = 'apt-get'
    values = (PIP, APT)
