# coding: utf-8

from __future__ import unicode_literals

import argparse
import logging
import os
import sys
from datetime import datetime

from tionix_autosetup.executors.base import AutosetupException
from tionix_autosetup.executors.package import DEFAULT_SOURCE
from tionix_autosetup.helpers import (
    create_argparser, OperationsEnum, suppress_stdout, get_version,
    save_installed_packages_info, save_used_pypi_host, BEFORE_AUTOSETUP_ENV,
    ManagerTypesEnum)
from tionix_autosetup.packages import PACKAGES
from tionix_autosetup.parser import ConnectionsCheckParser
from tionix_autosetup.pipeline import (
    process_package, get_variables_for_package, process_pipeline, get_pipeline)
from tionix_autosetup.variables import variables, ModuleBaseVar

logger = logging.getLogger(__name__)


def create_parser():
    """
    Parse arguments from command line and sys environment
    :return: dict with parsed args
    """
    description = '''
    TIONIX Auto setup script.
    Uses ENV variables for authentication
    '''

    epilog = '''
    Additionally, you can specify the variables for a specific module,
    using prefixes from set ({0}).
    Example: variable TNX_CC_DB_USERNAME override common (for all modules)
    variable DB_USERNAME separately for TIONIX Cloud Control.
    '''.format(', '.join(PACKAGES.prefixes.values()))

    parser = create_argparser(description=description,
                              epilog=epilog,
                              conflict_handler='resolve')

    def add_single_argument(names, prefix=None, **kwargs):
        for name in names:
            if prefix:
                argument = '--{0}_{1}'.format(prefix, name)
            else:
                argument = '--{0}'.format(name)
            parser.add_argument(argument, **kwargs)

    def add_common_argument(names, **kwargs):
        add_single_argument(names, **kwargs)
        _kwargs = dict(kwargs)
        _kwargs.update({'help': argparse.SUPPRESS})
        for _package, prefix in PACKAGES.prefixes.items():
            add_single_argument(names, prefix=prefix, **_kwargs)

    def get_variable_parameters(variable):
        parameters = dict(required=False, type=str)
        if variable.__doc__:
            if isinstance(variable.default, (tuple, list)):
                default_value = ', '.join(variable.default)
            else:
                default_value = variable.default
            parameters['help'] = (
                variable.__doc__.strip() +
                screening_params(" (DEFAULT:{0})" .format(default_value)))

        extraction = getattr(variable, 'extraction', None)

        if variable.is_collection:
            parameters['nargs'] = '+'

        names = []
        if extraction:
            if isinstance(extraction, basestring):
                names.append(extraction)
            else:
                for argument in list(extraction):
                    names.append(argument)
        elif hasattr(variable, 'name'):
            names.append(variable.name)
        return names, parameters

    def parse_variables(vars):
        for variable in vars:
            names, parameters = get_variable_parameters(variable)
            if issubclass(variable, ModuleBaseVar):
                # prefixes won't be prepended to module's unique variables
                add_single_argument(names, **parameters)
            else:
                add_common_argument(names, **parameters)

    vars = variables[:]

    module_variables = []
    # get modules unique variables
    for custom_variables in PACKAGES.custom_variables.values():
        module_variables.extend([
            variable for variable in custom_variables.values()
            if issubclass(variable, ModuleBaseVar)])
    vars.extend(module_variables)

    parse_variables(vars)

    parser.add_argument(
        '--config_file',
        required=False,
        type=str,
        default=None,
        help='Config file for autosetup',
    )

    package_choices = PACKAGES.active[:]
    package_choices.remove(PACKAGES.TIONIX)

    parser.add_argument(
        '--packages',
        nargs='+',
        choices=package_choices,
        required=False,
        type=str,
        default=package_choices,
        help='option to select packages for installation. By default installs '
             'all packages.',
    )

    action_group = parser.add_mutually_exclusive_group()

    action_group.add_argument(
        '--update',
        action='store_true',
        required=False,
        default=False,
        help='option to update installed packages',
    )

    action_group.add_argument(
        '--delete',
        action='store_true',
        required=False,
        default=False,
        help='option to delete installed packages',
    )

    action_group.add_argument(
        '--rollback',
        action='store_true',
        required=False,
        default=False,
        help='option to rollback packages versions',
    )

    parser.add_argument(
        '--file_path',
        type=str,
        required=False,
        default=None,
        help='option to set file path to rollback pip environment',
    )

    parser.add_argument(
        '--force',
        action='store_true',
        required=False,
        default=False,
        help='option to force delete installed packages',
    )

    parser.add_argument(
        '--no-check-connection',
        action='store_true',
        required=False,
        default=False,
        help='option to skip check the availability '
             'of all services before run',
    )

    parser.add_argument(
        '--verbose',
        action='store_true',
        required=False,
        default=False,
        help='option to enable detailed output'
    )

    parser.add_argument(
        '--cloud-domain-policy',
        action='store_true',
        required=False,
        default=False,
        help='option to setup/unset cloud domain policy',
    )

    parser.add_argument(
        '--overwrite-configs',
        action='store_true',
        required=False,
        default=False,
        help='option to overwrite modules yaml configs',
    )

    parser.add_argument(
        '--source',
        choices=ManagerTypesEnum.values,
        type=str,
        required=False,
        default=DEFAULT_SOURCE,
        help='source of modules installation',
    )

    return parser


def screening_params(variable):
    return variable.replace('%', '%%')


def root_check():
    """
        Purpose: root_check(): checks if the user has admins permission
        Effects: prints out a message and adds a record to the log
    """
    if os.geteuid() != 0:
        logger.info("Program finished with an error.\n"
                    "User doesn't have the required permissions")
        exit("You need to have root privileges to run this script.\n"
             "Please try again, this time using 'sudo'. \nExiting.")


def check_connections(env_vars, packages):
    """Checking services connection"""

    connections_check_parser = ConnectionsCheckParser()

    for package in packages:
        logger.info('Processing package {0}. '
                    'Operation: connections check'.format(package))
        with suppress_stdout():
            variable_box = get_variables_for_package(package, **env_vars)
            connections_check_pipeline = get_pipeline(
                OperationsEnum.CHECK_CONNECTION, package)
            results = process_pipeline(connections_check_pipeline,
                                       variable_box, package)
            for command_exec, result in results.items():
                connections_check_parser.add_result(command_exec, result,
                                                    package)

    successful_check = connections_check_parser.was_successful
    if not successful_check:
        logger.error("\n" + connections_check_parser.get_results())
    return successful_check


def pre_check(no_check_connection, force, env_vars, packages):

    # Check superuser access
    root_check()

    # Checking connections to services
    if not no_check_connection:
        logger.debug('Starting connection check for services')
        if check_connections(env_vars, packages):
            logger.debug('All services are available')
        else:
            logger.error('There are error in connection to services, '
                         'please check params')
            if not force:
                sys.exit(os.EX_SOFTWARE)


def call_pipeline(env_vars, packages, operation):
    """Process pipelines for installed modules"""
    for package in packages:
        variable_box = get_variables_for_package(package, **env_vars)
        pipeline = get_pipeline(operation, package)
        if pipeline:
            operation = operation.replace('_', ' ')
            logger.info('Processing package {0}. '
                        'Operation: {1}'.format(package, operation))
            process_pipeline(pipeline, variable_box, package)


def remove_empty(arguments):
    for arg, value in arguments.items():
        if value is None:
            arguments.pop(arg)


def check_args_usages(args, parser):
    force = args.get("force")
    delete = args.get("delete")
    overwrite_configs = args.get("overwrite_configs")
    cloud_domain_policy = args.get("cloud_domain_policy")
    no_check_connection = args.get("no_check_connection")
    rollback = args.get("rollback")
    config_file = args.get("config_file")
    admin_domain_id = args.get("ADMIN_DOMAIN_ID")
    file_path = args.get("file_path")

    if overwrite_configs and delete:
        parser.error('argument --overwrite_configs not allowed with argument '
                     '--delete')

    if force and not delete:
        parser.error('argument --force allowed only with argument --delete')

    if cloud_domain_policy and not admin_domain_id and not delete:
        parser.error('argument --cloud_domain_policy must be used '
                     'with argument --ADMIN_DOMAIN_ID if --delete is not set')

    if rollback and any([force, cloud_domain_policy, no_check_connection,
                         config_file]):
        parser.error('argument --rollback must be used only '
                     'with argument --verbose or --PYPI_HOST')

    if file_path and not rollback:
        parser.error('argument --file_path allowed only with '
                     'argument --rollback')


def get_env_vars():
    parser = create_parser()
    parsed_args = parser.parse_args()
    kwargs_dict = vars(parsed_args)
    remove_empty(kwargs_dict)
    check_args_usages(kwargs_dict, parser)
    return kwargs_dict


def main():
    logger.info('Starting TIONIX.Autosetup.'
                '{0} at {1}'.format(get_version(), datetime.now()))

    env_vars = get_env_vars()

    # log level for tionix autosetup
    verbose = env_vars.pop("verbose")
    log_level = logging.DEBUG if verbose else logging.INFO
    logging.getLogger().setLevel(log_level)

    force = env_vars.get("force")

    # Update package sign
    update_packages = env_vars.pop("update")
    # Deleting packages
    delete_packages = env_vars.pop("delete")

    if update_packages:
        operation = OperationsEnum.UPDATE
    elif delete_packages:
        operation = OperationsEnum.UNINSTALL
    else:
        operation = OperationsEnum.INSTALL

    # Changing packages
    packages = env_vars.pop("packages")

    # Add TIONIX to the end if uninstalling packages
    index = len(packages) if operation == OperationsEnum.UNINSTALL else 0
    packages.insert(index, PACKAGES.TIONIX)

    logger.debug('Args are:\n{0}'.format(env_vars))
    logger.debug('Env variables are:\n{0}'.format(
        '\n'.join([str({k: v}) for k, v in os.environ.items()])))

    rollback = env_vars.pop("rollback")
    aborted = False
    if rollback:
        call_pipeline(env_vars, packages, OperationsEnum.ROLLBACK)
        logger.info('Packages versions rollback completed')
    else:
        before_env = BEFORE_AUTOSETUP_ENV.format(datetime.now().strftime(
            u'%d%m%Y_%H%M%S'))
        save_installed_packages_info(before_env)
        if operation in (OperationsEnum.INSTALL, OperationsEnum.UPDATE):
            variable_box = get_variables_for_package(PACKAGES.TIONIX,
                                                     **env_vars)
            save_used_pypi_host(variable_box)

        # Checking connections to services
        no_check_connection = env_vars.pop("no_check_connection")

        pre_check(no_check_connection, force, env_vars, packages)

        for package in packages:
            try:
                process_package(package, operation, env_vars, force=force)
            except AutosetupException as err:
                logger.error(err)
                aborted = True
                break

    if not aborted:
        call_pipeline(env_vars, packages, OperationsEnum.POST_CHECK)


if __name__ == '__main__':
    main()
