# coding: utf-8

from __future__ import unicode_literals

import logging

from tionix_autosetup import variables
from tionix_autosetup.executors.base import AutosetupException
from tionix_autosetup.helpers import OperationsEnum
from tionix_autosetup.packages import PACKAGES

logger = logging.getLogger(__name__)


def get_variables_for_package(package, **kwargs):
    """ Parse available package env variables
    :param package: package name
    :param kwargs: params for VariableBox initialization
    :return: instance of tionix_autosetup.variables:VariableBox
    """

    fabric = PACKAGES.variable_box_fabric[package]
    # WARNING
    # order for update_by_instance reflecting on result values for variables
    variable = fabric(variables.DefaultsVariableBox, **kwargs)
    variable.update_by_instance(fabric(variables.SysEnvVariableBox, **kwargs))

    if kwargs.get("overwrite_configs"):
        kwargs['config_file'] = PACKAGES.config_path[package]
        variable.update_by_instance(fabric(variables.YamlFileBox, **kwargs))

    variable.update_by_instance(fabric(variables.ArgsVariableBox, **kwargs))
    variable.update_by_instance(fabric(variables.FileVariableBox, **kwargs))

    return variable


def process_pipeline(pipeline, variable_box, package, force=False):
    """ Run all sequence of task in pipeline
    :param pipeline: iterable of callable objects
    :param variable_box: variables
    :param package: processing package name
    :param force: flag to continue if executor processing failed
    :return results: dict with executors results
    :type variable_box: tionix_autosetup.variables.VariableBox
    """
    results = {}
    for executor in pipeline:
        executor_instance = executor(variable_box)
        result = executor_instance.run()
        results[executor] = result

        if executor_instance.critical_operation:
            _results = result
            if not isinstance(result, (list, tuple)):
                _results = [result]
            if any(_result.failed() for _result in _results):
                msg = '{0} package processing aborted.'.format(package)
                if not force:
                    raise AutosetupException(msg)
    return results


def get_pipeline(operation, package):
    if operation == OperationsEnum.INSTALL:
        pipelines = getattr(PACKAGES, 'pipelines', None)
    else:
        attr_name = '{0}_pipelines'.format(operation)
        pipelines = getattr(PACKAGES, attr_name, None)
    if pipelines:
        return pipelines.get(package)


def process_package(package, operation, env_vars, force=False):
    logger.info('Processing package {0}. Operation: {1}'.format(
        package, operation))
    variable_box = get_variables_for_package(package, **env_vars)

    pipeline = get_pipeline(operation, package)
    if pipeline:
        process_pipeline(pipeline, variable_box, package, force=force)
