# coding: utf-8

from __future__ import unicode_literals

import tempfile
import unittest

import os
import yaml

import mock

from tionix_autosetup.executors.mysql import MysqlCommands
from tionix_autosetup.executors.pip_service import PIPInstall, PIPUpdate
from tionix_autosetup.executors.rabbit import RabbitCommands
from tionix_autosetup.executors import config
from tionix_autosetup.packages import PACKAGES
from tionix_autosetup.pipeline import get_variables_for_package
from tionix_autosetup.tionix.configure import UpdateLocaleVariable
from tionix_autosetup.variables import DefaultsVariableBox, PipArgsVar


class TestExecutors(unittest.TestCase):
    package = PACKAGES.TIONIX_CC

    @classmethod
    def setUpClass(cls):
        super(TestExecutors, cls).setUpClass()
        current_dir_path = os.path.dirname(os.path.abspath(__file__))
        cls.templates_dir_path = os.path.join(current_dir_path, 'templates')

    env_data = {
        'DB_HOST': 'controller',
        'DB_NAME': 'test_db',
        'DB_USERNAME': 'test_db_user',
        'DB_SUPERUSER_NAME': 'test_user',
        'DB_SUPERUSER_PASSWORD': 'P@ssw0rd',
        'DB_ALLOWED_HOSTS': ('%',),
        'DB_USER_PASSWORD': 'P@ssw0rd',
    }

    def test_mysql(self):
        result = (
            u'mysql --user=test_user --password=P@ssw0rd --host=controller '
            u'-e "CREATE DATABASE test_db;"',
            u'mysql --user=test_user --password=P@ssw0rd --host=controller '
            u'-e "GRANT ALL PRIVILEGES ON test_db.* TO '
            u'\'test_db_user\'@\'%\' IDENTIFIED BY \'P@ssw0rd\';"',
            u'mysql --user=test_user --password=P@ssw0rd --host=controller -e '
            u'"GRANT ALL PRIVILEGES ON test_db.* TO '
            u'\'test_db_user\'@\'localhost\' IDENTIFIED BY \'P@ssw0rd\';"',
            u'mysql --user=test_user --password=P@ssw0rd --host=controller -e '
            u'"GRANT ALL PRIVILEGES ON test_db.* TO '
            u'\'test_db_user\'@\'controller\' IDENTIFIED BY \'P@ssw0rd\';"'
        )
        var_box = get_variables_for_package(
            PACKAGES.TIONIX_Scheduler, **self.env_data)
        mysql_exec = MysqlCommands(var_box)
        mysql_exec.sh_exec = lambda cmd, safe_cmd: cmd
        self.assertItemsEqual(result, mysql_exec.run().run_result)

    def test_rabbit(self):

        var_box = get_variables_for_package(
            PACKAGES.TIONIX_CC, **self.env_data)
        var_box_dict = var_box.to_dict()
        r_exec = RabbitCommands(var_box)
        r_exec.sh_exec = lambda cmd, safe_cmd: cmd

        result = []
        for s in RabbitCommands.commands:
            result.append(s.format(**var_box_dict))

        self.assertEqual(result, r_exec.run().run_result)

    def test_config(self):
        """Creates config file and fill it,
        checks result of variable extraction and formating the config file
        """
        expected_result = {'TNX_USERNAME': 'test_user'}
        variable_box = DefaultsVariableBox(**expected_result)

        _id, tmp_file = tempfile.mkstemp(prefix='tnx_tmp_', suffix='.yaml')

        class TestCloudControlConfig(config.Config):
            config_path = tmp_file
            template_name = 'test1.template'
            templates_dir_path = self.templates_dir_path

        os.remove(tmp_file)
        TestCloudControlConfig(variable_box).run()

        with open(tmp_file) as fh:
            yaml_dict = yaml.load(fh)

        self.assertEqual(expected_result, yaml_dict)
        os.remove(tmp_file)

    def test_config_exist(self):
        """If config file exists than it should not be changed"""
        data = {'TNX_USERNAME': 'test_user', 'KEY': 'VALUE'}
        variable_box = DefaultsVariableBox(**data)

        _id, tmp_file = tempfile.mkstemp(prefix='tnx_tmp_', suffix='.yaml')

        with open(tmp_file, 'w') as fh:
            fh.write(yaml.dump(data, default_flow_style=True))

        class TestCloudControlConfig(config.Config):
            config_path = tmp_file
            template_name = 'test1.template'
            templates_dir_path = self.templates_dir_path

        TestCloudControlConfig(variable_box).run()

        with open(tmp_file) as fh:
            yaml_dict = yaml.load(fh)

        self.assertEqual(data, yaml_dict)
        os.remove(tmp_file)

    def test_config_rewrite(self):
        """If config file exists and `rewrite` attribute is True than it
        should be rewritten.
        """

        data = {'TNX_USERNAME': 'test_user', 'KEY': 'VALUE'}
        expected_result = {'TNX_USERNAME': 'test_user'}

        variable_box = DefaultsVariableBox(**data)

        _id, tmp_file = tempfile.mkstemp(prefix='tnx_tmp_', suffix='.yaml')

        with open(tmp_file, 'w') as fh:
            fh.write(yaml.dump(data, default_flow_style=True))

        class TestRewriteConfig(config.Config):
            config_path = tmp_file
            template_name = 'test1.template'
            templates_dir_path = self.templates_dir_path
            rewrite = True

        TestRewriteConfig(variable_box).run()

        with open(tmp_file) as fh:
            yaml_dict = yaml.load(fh)

        self.assertEqual(expected_result, yaml_dict)
        os.remove(tmp_file)

    def test_update_locale_variables(self):
        lc_all_old = os.getenv('LC_ALL') or ''
        lang = os.getenv('LANG')

        os.environ['LC_ALL'] = ''

        self.assertFalse(os.getenv('LC_ALL'))

        local_var_exec = UpdateLocaleVariable()
        local_var_exec.run()

        self.assertEqual(lang, os.getenv('LC_ALL'))

        os.environ['LC_ALL'] = lc_all_old

        self.assertEqual(lc_all_old, os.getenv('LC_ALL'))

    def test_pip_args(self):
        arg = '--pre'
        env_data = {
            PipArgsVar.name: arg,
        }

        var_box = get_variables_for_package(
            PACKAGES.TIONIX, **env_data)
        test_packages = ['test-package']
        for executor in (PIPInstall, PIPUpdate):
            pip_executor = executor(var_box, packages=test_packages)
            pip_executor.pre_check = mock.Mock(return_value=True)
            pip_executor.post_check = mock.Mock(return_value=True)
            mocked_sh_exec = mock.Mock()
            pip_executor.sh_exec = mocked_sh_exec

            expected_calls = len(test_packages)
            if not pip_executor.updated:
                expected_calls += 1

            pip_executor.run()
            self.assertEqual(mocked_sh_exec.call_count, expected_calls)

            # pylint: disable=unsubscriptable-object
            called_cmd = mocked_sh_exec.call_args[1].get('cmd')
            self.assertIn(arg, called_cmd)
