# coding: utf-8

import unittest
import pkg_resources

import mock

from tionix_autosetup import helpers


class VersionTestCase(unittest.TestCase):
    class MockDistribution(object):
        version = u'1.0.0'

    def setUp(self):
        self.get_distribution = pkg_resources.get_distribution
        pkg_resources.get_distribution = mock.Mock(
            return_value=self.MockDistribution)

    def test_get_version_returns_value(self):
        version = helpers.get_version()
        self.assertEqual(version, self.MockDistribution.version)

    def tearDown(self):
        pkg_resources.get_distribution = self.get_distribution


class VersionInvalidTestCase(unittest.TestCase):
    def setUp(self):
        self.get_package_name = helpers.get_package_name
        helpers.get_package_name = mock.Mock(
            return_value=u'invalid_package_name')

    def test_get_version_returns_none(self):
        version = helpers.get_version()
        self.assertIsNone(version)

    def tearDown(self):
        helpers.get_package_name = self.get_package_name


class VersionConflictTestCase(unittest.TestCase):
    def setUp(self):
        self.get_distribution = pkg_resources.get_distribution
        exception = pkg_resources.VersionConflict(u'Testing version conflict')
        pkg_resources.get_distribution = mock.Mock(side_effect=exception)

    def test_version_conflict(self):
        version = helpers.get_version()
        self.assertIsNone(version)

    def tearDown(self):
        pkg_resources.get_distribution = self.get_distribution


class DistributionNotFoundTestCase(unittest.TestCase):
    def setUp(self):
        self.get_distribution = pkg_resources.get_distribution
        exception = pkg_resources.DistributionNotFound(
            u'Testing distribution not found')
        pkg_resources.get_distribution = mock.Mock(side_effect=exception)

    def test_version_conflict(self):
        version = helpers.get_version()
        self.assertIsNone(version)

    def tearDown(self):
        pkg_resources.get_distribution = self.get_distribution


class PackageNameTestCase(unittest.TestCase):
    package_name = 'tionix-autosetup'

    def test_get_package_name(self):
        self.assertEqual(self.package_name, helpers.get_package_name())

    def test_invalid_package_name(self):
        # parsing wrong paths
        self.assertIsNone(helpers.get_package_name('/'))
        self.assertIsNone(helpers.get_package_name('/path/'))
        self.assertIsNone(helpers.get_package_name('/path/module'))
        # parsing right path
        self.assertEqual('tionix-module',
                         helpers.get_package_name('/path/tionix_module'))
        self.assertEqual('tionix-module',
                         helpers.get_package_name('/path/tionix_module/'))
        self.assertEqual('tionix-module-test',
                         helpers.get_package_name('/path/tionix_module_test'))


class LinuxDistributionTestCase(unittest.TestCase):

    def setUp(self):
        self.linux_distribution_orig = helpers.linux_distribution

    def tearDown(self):
        helpers.linux_distribution = self.linux_distribution_orig

    def test_is_alt_linux(self):
        helpers.linux_distribution = lambda *args: (
            'ALT Linux ', '7.0.5', 'silo')
        helpers.Platform._distribution = None
        self.assertEqual(helpers.Platform.distribution,
                         helpers.Platform.ALT_LINUX)

    def test_is_ubuntu(self):
        helpers.linux_distribution = lambda *args: (
            'Ubuntu', '16.04', 'xenial')
        helpers.Platform._distribution = None
        self.assertEqual(helpers.Platform.distribution,
                         helpers.Platform.UBUNTU)

    def test_is_invalid(self):
        helpers.linux_distribution = lambda *args: (
            'Invalid distr', '16.04', 'xenial')
        helpers.Platform._distribution = None
        self.assertEqual(helpers.Platform.distribution,
                         helpers.Platform.DEFAULT)


class FillMissingKeysTest(unittest.TestCase):
    """Checks helpers.fill_missing_keys function"""
    orig_dict = {
        "DB": {
            "name": "MySQL",
            "version": "14.2.2",
        },
        "NOVA_VERSION": 2,
        "DICT_OF_NESTED_DICTS": {
            "VALUE_ONE": {
                "VALUE_OF_VALUE": 3,
            },
            "VALUE_TWO": {
                "OTHER_DICT": {
                }
            }
        }
    }
    etalon_dict = {
        "DB": {
            "name": "TestSQL",
            "version": "14.2.2",
            'user': 'tester'
        },
        "NOVA_VERSION": 3,
        "LOG_LEVEL": 'DEBUG',
        "DICT_OF_NESTED_DICTS": {
            "VALUE_ONE": {
                "VALUE_OF_VALUE": 1,
                "OTHER_VALUE_OF_VALUE": 2,
            },
            "VALUE_TWO": {
                "OTHER_DICT": {
                    "ANOTHER_DICT": {}
                }
            }
        }
    }

    def check_nested_keys_are_equal(self, test_dict, etalon_dict):
        self.assertItemsEqual(sorted(test_dict.keys()),
                              sorted(etalon_dict.keys()))

        for key, value in etalon_dict.items():
            if isinstance(value, dict):
                self.check_nested_keys_are_equal(test_dict[key], value)

    def check_orig_values_not_modified(self, test_dict, orig_dict):
        for key, value in orig_dict.items():
            if isinstance(value, dict):
                self.check_orig_values_not_modified(test_dict[key], value)
            else:
                self.assertEqual(test_dict[key], value)

    def check_new_keys_values_updated(self, test_dict, etalon_dict, orig_dict):
        updated_keys = etalon_dict.viewkeys() - orig_dict.viewkeys()

        for key, value in test_dict.items():
            if isinstance(value, dict) and key not in updated_keys:
                self.check_new_keys_values_updated(value,
                                                   etalon_dict[key],
                                                   orig_dict[key])
            else:
                if key in updated_keys:
                    self.assertEqual(value, etalon_dict[key])

    def test_add_missing_key(self):
        """Test that function adds missing keys"""
        updated_dict = helpers.fill_missing_keys(self.orig_dict,
                                                 self.etalon_dict)

        self.check_nested_keys_are_equal(updated_dict, self.etalon_dict)

    def test_does_not_update_current_values(self):
        """Test that function doesn't modify old values"""
        updated_dict = helpers.fill_missing_keys(self.orig_dict,
                                                 self.etalon_dict)

        self.check_orig_values_not_modified(updated_dict, self.orig_dict)

    def test_correctly_set_new_key_values(self):
        """Test that function correctly add new key values"""
        updated_dict = helpers.fill_missing_keys(self.orig_dict,
                                                 self.etalon_dict)

        self.check_new_keys_values_updated(updated_dict, self.etalon_dict,
                                           self.orig_dict)

    def test_orig_dict_has_extra_keys(self):
        """Test that function works correct with unknown keys."""
        orig_dict_with_extra_key = self.orig_dict.copy()
        orig_dict_with_extra_key['unknown'] = {"some_value": 3}

        updated_dict = helpers.fill_missing_keys(orig_dict_with_extra_key,
                                                 self.etalon_dict)
        self.assertDictEqual(updated_dict['unknown'],
                             orig_dict_with_extra_key['unknown'])
