# coding: utf-8

from __future__ import unicode_literals

import sys
import unittest

import mock

import tionix_autosetup.main as main

from tionix_autosetup.main import (OperationsEnum, create_parser,
                                   check_args_usages)
from tionix_autosetup.packages import PACKAGES
from tionix_autosetup.pipeline import get_pipeline
from tionix_autosetup.variables import variables, ModuleBaseVar


class PackageInstallation(unittest.TestCase):
    def setUp(self):
        self.parser = create_parser()

    def test_default_installation(self):
        """
        Checks if all packages are installed by default
        """
        parsed = self.parser.parse_args('')
        self.assertEqual(parsed.packages, PACKAGES.active[1:])

    def test_custom_installation(self):
        """
        Checks selective package installation
        """
        for package in PACKAGES.active[1:]:
            parsed = self.parser.parse_args(['--packages', package])
            self.assertEqual(parsed.packages, [package])

    def test_get_pipeline(self):
        for package in PACKAGES.active:
            self.assertEqual(
                get_pipeline(OperationsEnum.INSTALL, package),
                PACKAGES.pipelines.get(package)
            )

            self.assertEqual(
                get_pipeline(OperationsEnum.UPDATE, package),
                PACKAGES.update_pipelines.get(package)
            )


class UpdatePackagesTestCase(unittest.TestCase):

    @mock.patch('tionix_autosetup.main.root_check',
                new=mock.Mock(return_value=True))
    @mock.patch('tionix_autosetup.pipeline.process_pipeline')
    @mock.patch('tionix_autosetup.main.get_env_vars')
    def test_update_single_package(self, mock_env_vars, mock_process_pipeline):
        packages_list = [PACKAGES.TIONIX_Dashboard]
        fake_vars = {
            'packages': packages_list,
            'update': True,
            'delete': False,
            'no_check_connection': True,
            'verbose': False,
            'rollback': False,
            'cloud_domain_policy': False,
            'overwrite_configs': False,
        }
        mock_env_vars.return_value = fake_vars
        # packages_list will be modified in main.main
        main.main()

        self.assertEqual(mock_process_pipeline.call_count,
                         len(packages_list))

        called_pipelines = [args[0] for args, _kwargs in
                            mock_process_pipeline.call_args_list]
        pipeline = PACKAGES.update_pipelines[PACKAGES.TIONIX_Dashboard]
        self.assertIn(pipeline, called_pipelines)

    @mock.patch('tionix_autosetup.main.root_check',
                new=mock.Mock(return_value=True))
    @mock.patch('tionix_autosetup.pipeline.process_pipeline')
    @mock.patch('tionix_autosetup.main.get_env_vars')
    def test_update_all_packages(self, mock_env_vars, mock_process_pipeline):
        fake_vars = {
            'packages': PACKAGES.active[1:],
            'update': True,
            'delete': False,
            'rollback': False,
            'no_check_connection': True,
            'verbose': False,
            'cloud_domain_policy': False,
            'overwrite_configs': False,
        }
        mock_env_vars.return_value = fake_vars

        main.main()

        self.assertEqual(mock_process_pipeline.call_count,
                         len(PACKAGES.update_pipelines))

        called_pipelines = [args[0] for args, _kwargs in
                            mock_process_pipeline.call_args_list]

        for pipeline in PACKAGES.update_pipelines.values():
            self.assertIn(pipeline, called_pipelines)


class DeletePackagesTestCase(unittest.TestCase):
    @mock.patch('tionix_autosetup.main.root_check',
                new=mock.Mock(return_value=True))
    @mock.patch('tionix_autosetup.pipeline.process_pipeline')
    @mock.patch('tionix_autosetup.main.get_env_vars')
    def test_delete_single_package(self, mock_env_vars, mock_process_pipeline):
        fake_vars = {
            'packages': [PACKAGES.TIONIX_Dashboard],
            'delete': True,
            'update': False,
            'force': True,
            'rollback': False,
            'no_check_connection': True,
            'verbose': False,
            'cloud_domain_policy': False,
            'overwrite_configs': False,
        }
        mock_env_vars.return_value = fake_vars

        main.main()

        self.assertEqual(mock_process_pipeline.call_count, 2)

        called_pipelines = [args[0] for args, _kwargs in
                            mock_process_pipeline.call_args_list]
        pipeline = PACKAGES.uninstall_pipelines[PACKAGES.TIONIX_Dashboard]
        self.assertIn(pipeline, called_pipelines)

    @mock.patch('tionix_autosetup.main.root_check',
                new=mock.Mock(return_value=True))
    @mock.patch('tionix_autosetup.pipeline.process_pipeline')
    @mock.patch('tionix_autosetup.main.get_env_vars')
    def test_delete_all_packages(self, mock_env_vars, mock_process_pipeline):
        fake_vars = {
            'packages': PACKAGES.active[1:],
            'delete': True,
            'update': False,
            'force': True,
            'rollback': False,
            'no_check_connection': True,
            'verbose': False,
            'cloud_domain_policy': False,
            'overwrite_configs': False,
        }
        mock_env_vars.return_value = fake_vars

        main.main()

        self.assertEqual(mock_process_pipeline.call_count,
                         len(PACKAGES.uninstall_pipelines))

        called_pipelines = [args[0] for args, _kwargs in
                            mock_process_pipeline.call_args_list]

        for pipeline in PACKAGES.uninstall_pipelines.values():
            self.assertIn(pipeline, called_pipelines)

        self.assertEqual(called_pipelines[-1],
                         PACKAGES.uninstall_pipelines[PACKAGES.TIONIX])


@mock.patch('tionix_autosetup.main.root_check',
            new=mock.Mock(return_value=True))
@mock.patch('tionix_autosetup.main.check_connections')
@mock.patch('tionix_autosetup.pipeline.process_pipeline',
            new=mock.Mock(return_value=True))
@mock.patch('tionix_autosetup.main.get_env_vars')
class ConnectionsCheckTestCase(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_check_connections(self, mock_env_vars, mock_check):
        fake_vars = {
            'packages': [PACKAGES.TIONIX_CC],
            'delete': False,
            'update': False,
            'force': False,
            'rollback': False,
            'no_check_connection': False,
            'verbose': False,
            'cloud_domain_policy': False,
            'overwrite_configs': False,
        }
        mock_env_vars.return_value = fake_vars
        main.main()
        self.assertTrue(mock_check.called)

    def test_no_check_connections(self, mock_env_vars, mock_check):
        fake_vars = {
            'packages': [PACKAGES.TIONIX_CC],
            'delete': False,
            'update': False,
            'force': False,
            'rollback': False,
            'no_check_connection': True,
            'verbose': False,
            'cloud_domain_policy': False,
            'overwrite_configs': False,
        }
        mock_env_vars.return_value = fake_vars
        main.main()
        self.assertFalse(mock_check.called)


class ArgParseTest(unittest.TestCase):
    def setUp(self):
        self.parser = create_parser()
        self.argv = sys.argv
        self.maxDiff = None

    def tearDown(self):
        sys.argv = self.argv

    @staticmethod
    def _add_argument(arg_dict, arg):
        arg_dict.update(arg)
        for prefix in PACKAGES.prefixes.values():
            for key, value in arg.items():
                arg_dict.update({'{0}_{1}'.format(prefix, key): value})

    @staticmethod
    def _add_single_argument(arg_dict, arg):
        for key, value in arg.items():
            arg_dict.update({'{0}'.format(key): value})

    @staticmethod
    def _get_variable_names(variable):
        names = []

        extraction = getattr(variable, 'extraction', None)
        if extraction:
            if isinstance(extraction, basestring):
                names.append(variable.extraction)
            else:
                for argument in list(extraction):
                    names.append(argument)
        elif hasattr(variable, 'name'):
            names.append(variable.name)
        return names

    def test_parsed_arguments(self):
        """
        Testing parsed arguments
        """
        arguments_dict = dict()

        for variable in variables:
            names = self._get_variable_names(variable)
            parameter = None
            for name in names:
                self._add_argument(arguments_dict, {name: parameter})

        for custom_variables in PACKAGES.custom_variables.values():
            module_variables = [
                variable for variable in custom_variables.values()
                if issubclass(variable, ModuleBaseVar)]
            for variable in module_variables:
                names = self._get_variable_names(variable)
                parameter = None
                for name in names:
                    self._add_single_argument(
                        arguments_dict, {name: parameter})

        arguments_dict.update({
            'packages': PACKAGES.active[1:],
            'config_file': None,
            'update': False,
            'delete': False,
            'force': False,
            'rollback': False,
            'file_path': None,
            'no_check_connection': False,
            'verbose': False,
            'cloud_domain_policy': False,
            'overwrite_configs': False,
            'source': 'pip',
        })

        arguments_dict = {
            str(key): str(value) if isinstance(value, basestring) else value
            for key, value in arguments_dict.items()
        }

        parsed = self.parser.parse_args('')

        parsed_dict = dict(parsed._get_kwargs())
        self.assertEqual(parsed_dict, arguments_dict)

        def error_fn(message):
            raise ValueError(message)

        error_orig = self.parser.error
        self.parser.error = error_fn

        with self.assertRaises(ValueError):
            self.parser.parse_args(['--update', '--delete'])

        args = self.parser.parse_args(['--update', '--force'])
        with self.assertRaises(ValueError):
            check_args_usages(dict(args._get_kwargs()), self.parser)

        self.parser.error = error_orig
