# coding: utf-8

import unittest

import mock

from pkg_resources import DistributionNotFound

from tionix_autosetup.packages import PACKAGES
from tionix_autosetup.executors.package import Install
from tionix_autosetup.executors.manager import RPMInstall
from tionix_autosetup.executors.pip_service import (
    PIPInstall, PIPUninstall, PIPUpdate)
from tionix_autosetup.helpers import PackageTypesEnum
from tionix_autosetup.tionix_cloud_control.configure import (
    pipeline, uninstall_pipeline, update_pipeline)
from tionix_autosetup.pipeline import (
    get_variables_for_package, process_pipeline)


PIP_MODULE_NAME = 'test-module-name'
RPM_MODULE_NAME = 'python-test_module_name'


class InstallTestModule(Install):
    package_names = {
        PackageTypesEnum.PIP: PIP_MODULE_NAME,
        PackageTypesEnum.RPM: RPM_MODULE_NAME,
    }


class PIPInstallTestModule(PIPInstall):
    def __init__(self, *args, **kwargs):
        super(PIPInstallTestModule, self).__init__(*args, **kwargs)
        self.updated = True

    packages = PIP_MODULE_NAME


class RPMTestInstall(RPMInstall):
    packages = RPM_MODULE_NAME


class InstallationManagersTestCase(unittest.TestCase):

    def __init__(self, *args):
        super(InstallationManagersTestCase, self).__init__(*args)
        self.fake_vars = {
            'packages': [PACKAGES.TIONIX_CC],
            'update': False,
            'delete': False,
            'no_check_connection': True,
            'verbose': False,
            'rollback': False,
            'cloud_domain_policy': False,
            'overwrite_configs': False,
        }

    @mock.patch('tionix_autosetup.executors.base.Executor.run')
    @mock.patch('tionix_autosetup.executors.pip_service.PIPInstall.run')
    @mock.patch('tionix_autosetup.main.get_env_vars')
    def test_pip_installation(self, mock_env_vars, pip_run, run):

        fake_vars = self.fake_vars.copy()
        fake_vars['source'] = 'pip'
        mock_env_vars.return_value = fake_vars

        package = PACKAGES.TIONIX_CC

        result = mock.Mock()
        result.failed = mock.Mock(return_value=False)
        run.return_value = result
        pip_run.return_value = result

        variable_box = get_variables_for_package(package, **fake_vars)
        process_pipeline(pipeline, variable_box, package)

        pip_run.assert_called_once()

    @mock.patch('tionix_autosetup.executors.base.Executor.run')
    @mock.patch('tionix_autosetup.executors.pip_service.PIPUninstall.run')
    @mock.patch('tionix_autosetup.main.get_env_vars')
    def test_pip_uninstallation(self, mock_env_vars, pip_run, run):

        fake_vars = self.fake_vars.copy()
        fake_vars['source'] = 'pip'
        fake_vars['delete'] = True
        mock_env_vars.return_value = fake_vars

        package = PACKAGES.TIONIX_CC

        result = mock.Mock()
        result.failed = mock.Mock(return_value=False)
        run.return_value = result
        pip_run.return_value = result

        variable_box = get_variables_for_package(package, **fake_vars)
        process_pipeline(uninstall_pipeline, variable_box, package)

        pip_run.assert_called_once()

    @mock.patch('tionix_autosetup.executors.base.Executor.run')
    @mock.patch('tionix_autosetup.executors.pip_service.PIPUpdate.run')
    @mock.patch('tionix_autosetup.main.get_env_vars')
    def test_pip_update(self, mock_env_vars, pip_run, run):

        fake_vars = self.fake_vars.copy()
        fake_vars['source'] = 'pip'
        fake_vars['update'] = True
        mock_env_vars.return_value = fake_vars

        package = PACKAGES.TIONIX_CC

        result = mock.Mock()
        result.failed = mock.Mock(return_value=False)
        run.return_value = result
        pip_run.return_value = result

        variable_box = get_variables_for_package(package, **fake_vars)
        process_pipeline(update_pipeline, variable_box, package)

        pip_run.assert_called_once()

    @mock.patch('tionix_autosetup.executors.base.Executor.run')
    @mock.patch('tionix_autosetup.executors.pip_service.PIPInstall.run')
    @mock.patch('tionix_autosetup.executors.manager.RPMInstall.run')
    @mock.patch('tionix_autosetup.main.get_env_vars')
    def test_rpm_installation(self, mock_env_vars, apt_run, pip_run, run):

        fake_vars = self.fake_vars.copy()
        fake_vars['source'] = 'apt-get'
        mock_env_vars.return_value = fake_vars

        package = PACKAGES.TIONIX_CC

        result = mock.Mock()
        result.failed = mock.Mock(return_value=False)
        run.return_value = result
        pip_run.return_value = result
        apt_run.return_value = result

        variable_box = get_variables_for_package(package, **fake_vars)
        process_pipeline(pipeline, variable_box, package)

        pip_run.assert_not_called()
        apt_run.assert_called_once()

    @mock.patch('tionix_autosetup.executors.base.Executor.run')
    @mock.patch('tionix_autosetup.executors.pip_service.PIPInstall.run')
    @mock.patch('tionix_autosetup.executors.manager.RPMUninstall.run')
    @mock.patch('tionix_autosetup.main.get_env_vars')
    def test_rpm_uninstallation(self, mock_env_vars, apt_run, pip_run, run):

        fake_vars = self.fake_vars.copy()
        fake_vars['source'] = 'apt-get'
        fake_vars['delete'] = True
        mock_env_vars.return_value = fake_vars

        package = PACKAGES.TIONIX_CC

        result = mock.Mock()
        result.failed = mock.Mock(return_value=False)
        run.return_value = result
        pip_run.return_value = result
        apt_run.return_value = result

        variable_box = get_variables_for_package(package, **fake_vars)
        process_pipeline(uninstall_pipeline, variable_box, package)

        pip_run.assert_not_called()
        apt_run.assert_called_once()

    @mock.patch('tionix_autosetup.executors.base.Executor.run')
    @mock.patch('tionix_autosetup.executors.pip_service.PIPInstall.run')
    @mock.patch('tionix_autosetup.executors.manager.RPMUpdate.run')
    @mock.patch('tionix_autosetup.main.get_env_vars')
    def test_rpm_update(self, mock_env_vars, apt_run, pip_run, run):

        fake_vars = self.fake_vars.copy()
        fake_vars['source'] = 'apt-get'
        mock_env_vars.return_value = fake_vars

        package = PACKAGES.TIONIX_CC

        result = mock.Mock()
        result.failed = mock.Mock(return_value=False)
        run.return_value = result
        pip_run.return_value = result
        apt_run.return_value = result

        variable_box = get_variables_for_package(package, **fake_vars)
        process_pipeline(update_pipeline, variable_box, package)

        pip_run.assert_not_called()
        apt_run.assert_called_once()

    @mock.patch('tionix_autosetup.main.get_env_vars')
    def test_strategies_support_pip(self, mock_env_vars):
        fake_vars = self.fake_vars.copy()
        fake_vars['source'] = 'pip'
        mock_env_vars.return_value = fake_vars

        package = PACKAGES.TIONIX_CC

        variable_box = get_variables_for_package(package, **fake_vars)
        install_exec = InstallTestModule(variable_box)

        pip_exec = PIPInstallTestModule(variable_box)

        self.assertTrue(install_exec.commands)
        self.assertListEqual(install_exec.commands, pip_exec.commands)

    @mock.patch('tionix_autosetup.main.get_env_vars')
    def test_strategies_support_rpm(self, mock_env_vars):
        fake_vars = self.fake_vars.copy()
        fake_vars['source'] = 'apt-get'
        mock_env_vars.return_value = fake_vars

        package = PACKAGES.TIONIX_CC

        variable_box = get_variables_for_package(package, **fake_vars)
        install_exec = InstallTestModule(variable_box)

        apt_exec = RPMTestInstall(variable_box, package_manager='apt-get')

        self.assertTrue(install_exec.commands)
        self.assertListEqual(install_exec.commands, apt_exec.commands)

    @mock.patch('pkg_resources.get_distribution')
    @mock.patch('tionix_autosetup.executors.pip_service.reload',
                new=mock.Mock(return_value=None))
    def test_pip_install_checks(self, get_distribution):

        def raise_dict_not_found(*args, **kwargs):
            raise DistributionNotFound

        both_available = mock.Mock(return_value=None)
        both_unavailable = raise_dict_not_found
        pip_unavailable = (lambda _package: raise_dict_not_found()
                           if _package == 'pip' else None)
        pip_available = (lambda _package: None if _package == 'pip'
                         else raise_dict_not_found())

        fake_vars = self.fake_vars.copy()
        fake_vars['source'] = 'pip'

        package = PACKAGES.TIONIX_CC
        variable_box = get_variables_for_package(package, **fake_vars)

        # installation
        executor = PIPInstall(variable_box)
        executor.packages = (PIP_MODULE_NAME, )

        get_distribution.side_effect = both_available
        self.assertTrue(executor.pre_check())
        get_distribution.side_effect = both_unavailable
        self.assertFalse(executor.pre_check())

        get_distribution.side_effect = both_available
        self.assertTrue(executor.post_check())
        get_distribution.side_effect = both_unavailable
        self.assertFalse(executor.post_check())

        # uninstallation
        executor = PIPUninstall(variable_box)
        executor.packages = (PIP_MODULE_NAME, )

        get_distribution.side_effect = both_available
        self.assertTrue(executor.pre_check())
        get_distribution.side_effect = pip_available
        self.assertFalse(executor.pre_check())
        get_distribution.side_effect = pip_unavailable
        self.assertFalse(executor.pre_check())

        get_distribution.side_effect = both_available
        self.assertFalse(executor.post_check())
        get_distribution.side_effect = both_unavailable
        self.assertTrue(executor.post_check())
        get_distribution.side_effect = pip_available
        self.assertTrue(executor.post_check())

        # update
        executor = PIPUpdate(variable_box)
        executor.packages = (PIP_MODULE_NAME, )

        get_distribution.side_effect = both_available
        self.assertTrue(executor.pre_check())
        get_distribution.side_effect = pip_available
        self.assertFalse(executor.pre_check())
        get_distribution.side_effect = pip_unavailable
        self.assertFalse(executor.pre_check())
