# coding: utf-8

from __future__ import unicode_literals

import json
import os
import re
import shutil
import sys
import tempfile
import unittest

from datetime import datetime, timedelta

import yaml

import mock
from mock import patch

from tionix_autosetup import pipeline
from tionix_autosetup.executors.config import (
    read_config_file, ConfigWithSectionsParser, UpdateSudoers, Config)
from tionix_autosetup.executors.base import (
    AutosetupException, Dirs, Executor, ExecutorResult)
from tionix_autosetup.main import create_parser, remove_empty
from tionix_autosetup.packages import PACKAGES
from tionix_autosetup.tionix.configure import (
    UpdateNovaVersionConf, RestoreKeystoneConfig, rename_date_format,
    keystone_policy_cfg, ActualizeConfig)
from tionix_autosetup.tionix_dashboard.configure import (
    ImportDashConfig, RemoveImportDashConfig, CloudPolicyParameters)
from tionix_autosetup.tionix_vdi_server.configure import (
    UpdateNovaConf, VDIServerUpdateApacheConfig)
from tionix_autosetup.tionix_scheduler.configure import (
    SchedulerUpdateApacheConfig, UpdateConfigSchedulerBeat)
from tionix_autosetup.variables import DefaultsVariableBox


class TestUpdateFilePipelines(unittest.TestCase):

    def __init__(self, methodName='runTest'):
        super(TestUpdateFilePipelines, self).__init__(methodName)
        self.variable_box = self.get_variable_box()
        self.variable_box.update(**{'DB_NAME': 'test'})

    def setUp(self):
        _id, self.temp_file = tempfile.mkstemp(suffix='.yaml')
        for executor in (
                ImportDashConfig, RemoveImportDashConfig,
                VDIServerUpdateApacheConfig, SchedulerUpdateApacheConfig,
                CloudPolicyParameters):
            patch.object(executor, 'config_path',
                         new=self.temp_file).start()

    def tearDown(self):
        try:
            os.remove(self.temp_file)
        except OSError:
            pass
        patch.stopall()

    def init_file(self, data=None):
        data = data or ''
        with open(self.temp_file, 'w') as fh:
            fh.write(data)

    def get_file_data(self):
        with open(self.temp_file) as fh:
            data = fh.read()
        return data

    @staticmethod
    def get_variable_box(**kwargs):
        return DefaultsVariableBox(**kwargs)

    @staticmethod
    def process_executor(executor, variable_box):
        custom_pipeline = [
            executor,
        ]
        return pipeline.process_pipeline(custom_pipeline, variable_box, None)

    def test_update_nova_conf(self):
        _config_path = UpdateNovaConf.config_path
        UpdateNovaConf.config_path = self.temp_file
        self.process_executor(UpdateNovaConf, self.variable_box)
        UpdateNovaConf.config_path = _config_path
        data = self.get_file_data()
        for key, value in UpdateNovaConf.config_keys.items():
            self.assertIn(
                '{0} = {1}'.format(key, value),
                data)

    def test_update_apache_config(self):
        expected_result = {'TNX_USERNAME': 'test_user'}
        variable_box = self.get_variable_box(**expected_result)
        self.init_file("""
            Listen 10001

            <VirtualHost *:10001>

            WSGIDaemonProcess tionix_scheduler user=tionix group=tionix proc=3
            """)
        replaced_str = 'user={TNX_USERNAME} group={TNX_USERNAME}'.format(
            **expected_result)

        self.process_executor(SchedulerUpdateApacheConfig, variable_box)
        data = self.get_file_data()

        self.assertIn(replaced_str, data)

    def test_update_apache_config_regexp(self):
        pattern = SchedulerUpdateApacheConfig.user_pattern
        self.assertIsNotNone(pattern.search(
            'tionix_scheduler user=tionix group=tionix processes=3'
        ))
        self.assertIsNotNone(pattern.search(
            '\ntionix_scheduler user=test_user    group=test_user  processes=3'
        ))
        self.assertIsNotNone(pattern.search(
            'tionix_scheduler user=user123 group=user123 processes=3'
        ))

    def test_update_apache_config_no_file(self):
        os.remove(self.temp_file)
        try:
            self.process_executor(SchedulerUpdateApacheConfig,
                                  self.variable_box)
        except IOError:
            self.fail('SchedulerUpdateApacheConfig raised IOError')

    def test_update_vdi_apache_config(self):
        executor = VDIServerUpdateApacheConfig

        init_data = """
WSGIDaemonProcess user=test group=test processes=3
        """

        expected_data = """
WSGIDaemonProcess user=tionix group=tionix processes=3
        """
        with open(self.temp_file, 'w') as fh:
            fh.write(init_data)
        self.process_executor(executor, self.variable_box)
        with open(self.temp_file) as fh:
            data = fh.read()
        self.assertEqual(data, expected_data)

    def test_update_sudoers(self):
        expected_result = {'TNX_USERNAME': 'test_user'}
        variable_box = self.get_variable_box(**expected_result)
        self.init_file("""
            admin ALL=(ALL) NOPASSWD: ALL
            user ALL=(ALL) NOPASSWD: ALL
            """)
        search_str = UpdateSudoers.config_template.format(**expected_result)
        _config_path = UpdateSudoers.file_path
        _commands = UpdateSudoers.commands
        UpdateSudoers.file_path = self.temp_file
        UpdateSudoers.commands = ('echo "{0}" >> {1}'.format(
            UpdateSudoers.config_template, UpdateSudoers.file_path),)
        self.process_executor(UpdateSudoers, variable_box)
        data = self.get_file_data()
        UpdateSudoers.file_path = _config_path
        UpdateSudoers.commands = _commands
        self.assertIn(search_str, data)

    def test_update_sudoers_already_exists(self):
        expected_result = {'TNX_USERNAME': 'test_user'}
        variable_box = self.get_variable_box(**expected_result)
        search_str = UpdateSudoers.config_template.format(**expected_result)
        self.init_file(search_str)
        _config_path = UpdateSudoers.file_path
        _commands = UpdateSudoers.commands
        UpdateSudoers.file_path = self.temp_file
        UpdateSudoers.commands = ('echo "{0}" >> {1}'.format(
            UpdateSudoers.config_template, UpdateSudoers.file_path),)
        self.process_executor(UpdateSudoers, variable_box)
        data = self.get_file_data()
        UpdateSudoers.file_path = _config_path
        UpdateSudoers.commands = _commands
        self.assertEqual(data.count(search_str), 1)

    def test_dash_config_databases_update(self):
        initial_data = ImportDashConfig.config_string.format(
            **self.variable_box.to_dict())
        self.init_file(initial_data)
        test_data = ImportDashConfig.config_string.format(
            **self.variable_box.to_dict())
        self.process_executor(ImportDashConfig, self.variable_box)
        data = self.get_file_data()
        self.assertIn(test_data, data)

    def test_dash_config_databases_add(self):
        test_data = ImportDashConfig.config_string.format(
            **self.variable_box.to_dict())
        self.process_executor(ImportDashConfig, self.variable_box)
        data = self.get_file_data()
        self.assertIn(test_data, data)
        post_check = ImportDashConfig(self.variable_box).post_check()
        self.assertTrue(post_check)

    def test_dash_config_databases_delete(self):
        executor = RemoveImportDashConfig
        self.variable_box.kwargs['force'] = True
        dash_config = executor.config_string.format(
            **self.variable_box.to_dict())
        test_data = 'test config\n{0}'.format(dash_config)
        self.init_file(test_data)
        results = self.process_executor(
            executor, self.variable_box)
        result = results.values()[0]
        data = self.get_file_data()
        self.assertNotIn(dash_config, data)
        self.assertTrue(result.post_check)

    def test_dash_config_databases_delete_already_removed(self):
        test_data = 'test config'
        self.init_file(test_data)
        results = self.process_executor(
            RemoveImportDashConfig, self.variable_box)
        result = results.values()[0]
        self.assertFalse(result.pre_check)

    def test_dash_config_databases_exist(self):
        initial_data = ImportDashConfig.config_string.format(
            **self.variable_box.to_dict())
        self.init_file(initial_data)
        pre_check = ImportDashConfig(self.variable_box).pre_check()
        self.assertFalse(pre_check)

    def test_dash_config_databases_file_doesnt_exist(self):
        os.remove(self.temp_file)
        pre_check = ImportDashConfig(self.variable_box).pre_check()
        self.assertFalse(pre_check)

    def test_dash_config_databases_failed(self):
        executor = ImportDashConfig(self.variable_box)
        executor.data_for_process = {
            executor.config_string: (executor.make_pattern(
                executor.config_string), False)
        }
        post_check = executor.post_check()
        self.assertFalse(post_check)

    def test_dash_config_databases_regexp(self):
        executor = ImportDashConfig(self.variable_box)
        conf_str = executor.config_string.format(
            **self.variable_box.to_dict())
        self.assertRegexpMatches(conf_str,
                                 executor.new_config_data.get(conf_str))

    def test_dash_cloud_policy_regexp(self):
        executor = CloudPolicyParameters(self.variable_box)
        data = """
            OPENSTACK_KEYSTONE_DEFAULT_DOMAIN = "default"

            POLICY_FILES = {
                'identity': 'keystone_policy.v3cloudsample.json',
                'compute': 'nova_policy.json',
                'volume': 'cinder_policy.json',
                'image': 'glance_policy.json',
                'orchestration': 'heat_policy.json',
                'network': 'neutron_policy.json',
                'telemetry': 'ceilometer_policy.json',
            }

            SESSION_ENGINE = "django.contrib.sessions.backends.cache"

            CACHES = {
                'default': {
                    'BACKEND': 'django.core.cache.backends.memcached.*',
                    'LOCATION': "localhost",
                },
            }

            OPENSTACK_KEYSTONE_MULTIDOMAIN_SUPPORT = True
            #other data
            """

        for cfg, pattern in executor.new_config_data.items():
            cfg = cfg.format(**self.variable_box.to_dict())
            self.assertRegexpMatches(cfg, pattern)
            self.assertIsNotNone(re.search(pattern, data))

    def test_dash_cloud_policy_negative_regexp(self):
        executor = CloudPolicyParameters(self.variable_box)
        data = 'some test data'
        for _cfg, pattern in executor.new_config_data.items():
            self.assertIsNone(re.search(pattern, data))

    def test_dash_cloud_policy_params_add(self):
        kwargs = {}
        if hasattr(self.variable_box, 'kwargs'):
            kwargs = self.variable_box.kwargs
            self.variable_box.kwargs.update(cloud_domain_policy=True)
        executor = CloudPolicyParameters(self.variable_box)
        config_data = executor.new_config_data
        self.process_executor(CloudPolicyParameters, self.variable_box)
        data = self.get_file_data()
        for cfg, _pattern in config_data.items():
            cfg = cfg.format(**self.variable_box.to_dict())
            self.assertIn(cfg, data)
        post_check = CloudPolicyParameters(self.variable_box).post_check()
        self.assertTrue(post_check)
        self.variable_box.kwargs = kwargs

    def test_update_config_scheduler_beat(self):
        username = 'test_user'
        variable_box = self.get_variable_box(**{'TNX_USERNAME': username})
        self.init_file((
            "[program:tionix-scheduler-beat]\n"
            "directory = /tmp\n"
            "user = tionix\n"
            "group = tionix\n"))
        UpdateConfigSchedulerBeat.config_path = self.temp_file
        self.process_executor(UpdateConfigSchedulerBeat, variable_box)
        data = self.get_file_data()
        self.assertIn('user = {0}'.format(username), data)

    def test_update_config_no_file(self):
        ConfigWithSectionsParser.config_path = self.temp_file
        os.remove(self.temp_file)
        pre_check = ConfigWithSectionsParser(self.variable_box).pre_check()
        self.assertFalse(pre_check)

    def test_update_nova_config_no_section(self):
        UpdateNovaVersionConf.config_path = self.temp_file

        UpdateNovaVersionConf(self.variable_box).run()
        cfg_parser = read_config_file(UpdateNovaVersionConf.config_path)
        self.assertTrue(
            cfg_parser.has_section(UpdateNovaVersionConf.config_section))


class TestExtractor(unittest.TestCase):

    wrong_auth_url = 'wrong_auth_url'
    wrong_pass = 'wrong_auth_password'
    wrong_project = 'wrong_project_name'
    wrong_tnx_cc_pass = 'wrong_tnx_cc_pass'
    wrong_tnx_cc_com_api_ver = 'wrong_tnx_cc_com_api_ver'
    right_auth_url = 'right_auth_url'
    right_pass = 'right_auth_password'
    right_project = 'right_project_name'
    right_tnx_cc_pass = 'right_tnx_cc_pass'
    test_yaml_file_name = 'test_yaml_file.yaml'
    test_config_file_name = 'test_config_file.txt'
    test_cc_config_file_name = 'test_cc_config_file.txt'

    def __init__(self, methodName='runTest'):
        super(TestExtractor, self).__init__(methodName=methodName)
        self.test_dir = tempfile.mkdtemp()

    def setUp(self):
        super(TestExtractor, self).setUp()
        self.base_auth_url_env = os.getenv('AUTH_URL') or ''
        self.base_project_name_env = os.getenv('PROJECT_NAME') or ''
        self.base_os_pass_env = os.getenv('OS_PASSWORD') or ''
        self.base_tnx_cc_pass_env = os.getenv('TNX_CC_OS_PASSWORD') or ''
        self.yaml_file_fh = open(
            os.path.join(self.test_dir, self.test_yaml_file_name), 'w')
        self.config_file_fh = open(
            os.path.join(self.test_dir, self.test_config_file_name), 'w')
        self.cc_config_file_fh = open(
            os.path.join(self.test_dir, self.test_cc_config_file_name), 'w')

        current_dir_path = os.path.dirname(os.path.abspath(__file__))
        self.templates_dir_path = os.path.join(current_dir_path, 'templates')

    def tearDown(self):
        super(TestExtractor, self).tearDown()
        os.environ['AUTH_URL'] = self.base_auth_url_env
        os.environ['PROJECT_NAME'] = self.base_project_name_env
        os.environ['OS_PASSWORD'] = self.base_os_pass_env
        os.environ['TNX_CC_OS_PASSWORD'] = self.base_tnx_cc_pass_env
        os.remove(self.yaml_file_fh.name)
        os.remove(self.config_file_fh.name)
        os.remove(self.cc_config_file_fh.name)

    def test_pipeline(self):
        expected_result = {'TNX_USERNAME': 'test_user'}
        variable_box = DefaultsVariableBox(**expected_result)

        _id, tmp_file = tempfile.mkstemp(prefix='tnx_tmp_', suffix='.yaml')

        class TestCloudControlConfig(Config):
            config_path = tmp_file
            template_name = 'test1.template'
            templates_dir_path = self.templates_dir_path

        os.remove(tmp_file)
        custom_pipeline = [
            TestCloudControlConfig,
        ]

        pipeline.process_pipeline(custom_pipeline, variable_box, None)

        with open(tmp_file) as fh:
            yaml_dict = yaml.load(fh)

        self.assertEqual(expected_result, yaml_dict)
        os.remove(tmp_file)

    def test_critical_executors_aborted(self):

        variable_box = DefaultsVariableBox()
        test_count = 2

        class TestPreExecutor(Executor):
            run = mock.Mock()

        class TestPostExecutor(Executor):
            run = mock.Mock()

        class CriticalExecutor(Executor):
            critical_operation = True

            run = mock.Mock(return_value=ExecutorResult(
                pre_check=True, post_check=None,
                run_result=unicode(Exception()), run_check=False,
            ))

        pre_pipeline = test_count * [TestPreExecutor] + [CriticalExecutor]
        post_pipeline = test_count * [TestPostExecutor]
        test_pipeline = pre_pipeline + post_pipeline

        self.assertRaises(AutosetupException, pipeline.process_pipeline,
                          test_pipeline, variable_box, None)

        self.assertTrue(CriticalExecutor.run.called)
        self.assertEqual(TestPreExecutor.run.call_count, test_count)
        self.assertEqual(TestPostExecutor.run.call_count, 0)

    def test_critical_executors_aborted_force(self):

        variable_box = DefaultsVariableBox()
        test_count = 2

        class TestPreExecutor(Executor):
            run = mock.Mock()

        class TestPostExecutor(Executor):
            run = mock.Mock()

        class CriticalExecutor(Executor):
            critical_operation = True

            run = mock.Mock(return_value=ExecutorResult(
                pre_check=True, post_check=None,
                run_result=unicode(Exception()), run_check=False,
            ))

        pre_pipeline = test_count * [TestPreExecutor] + [CriticalExecutor]
        post_pipeline = test_count * [TestPostExecutor]
        test_pipeline = pre_pipeline + post_pipeline

        pipeline.process_pipeline(test_pipeline, variable_box, None,
                                  force=True)

        self.assertTrue(CriticalExecutor.run.called)
        self.assertEqual(TestPreExecutor.run.call_count, test_count)
        self.assertEqual(TestPostExecutor.run.call_count, test_count)

    def test_critical_executors_not_aborted(self):

        variable_box = DefaultsVariableBox()
        test_count = 2

        class TestPreExecutor(Executor):
            run = mock.Mock()

        class TestPostExecutor(Executor):
            run = mock.Mock()

        class CriticalExecutor(Executor):
            critical_operation = True

            run = mock.Mock(return_value=ExecutorResult(
                pre_check=True, post_check=True,
                run_result=None, run_check=True,
            ))

        pre_pipeline = test_count * [TestPreExecutor] + [CriticalExecutor]
        post_pipeline = test_count * [TestPostExecutor]
        test_pipeline = pre_pipeline + post_pipeline

        pipeline.process_pipeline(test_pipeline, variable_box, None)

        self.assertTrue(CriticalExecutor.run.called)
        self.assertEqual(TestPreExecutor.run.call_count, test_count)
        self.assertEqual(TestPostExecutor.run.call_count, test_count)

    def test_critical_executors_failed_not_critical(self):

        variable_box = DefaultsVariableBox()
        test_count = 2

        class TestPreExecutor(Executor):
            run = mock.Mock()

        class TestPostExecutor(Executor):
            run = mock.Mock()

        class NotCriticalExecutor(Executor):

            run = mock.Mock(return_value=ExecutorResult(
                pre_check=True, post_check=None,
                run_result=unicode(Exception()), run_check=False,
            ))

        pre_pipeline = test_count * [TestPreExecutor] + [NotCriticalExecutor]
        post_pipeline = test_count * [TestPostExecutor]
        test_pipeline = pre_pipeline + post_pipeline

        pipeline.process_pipeline(test_pipeline, variable_box, None)

        self.assertTrue(NotCriticalExecutor.run.called)
        self.assertEqual(TestPreExecutor.run.call_count, test_count)
        self.assertEqual(TestPostExecutor.run.call_count, test_count)

    def test_dirs_pipeline(self):
        variable_box = DefaultsVariableBox()
        tmp_file = tempfile.mkdtemp()

        class TestDirs(Dirs):
            dirs = [
                tmp_file,
                tmp_file + '_new']

        custom_pipeline = [
            TestDirs,
        ]

        pipeline.process_pipeline(custom_pipeline, variable_box, None)

        for cur_dir in TestDirs.dirs:
            self.assertTrue(os.path.exists(cur_dir))

        map(os.rmdir, TestDirs.dirs)

    def test_get_variables_for_package_with_yaml_file(self):
        yaml_file_data = (
            'OS_AUTH_URL: {0}'.format(self.wrong_auth_url))
        self.yaml_file_fh.write(yaml_file_data)
        self.yaml_file_fh.flush()

        kwargs = {'config_file': self.yaml_file_fh.name}

        sys.argv.append('--OS_AUTH_URL')
        sys.argv.append(self.right_auth_url)
        sys.argv.append('--OS_PASSWORD')
        sys.argv.append(self.right_pass)
        sys.argv.append('--OS_PROJECT_NAME')
        sys.argv.append(self.right_project)

        os.environ['OS_AUTH_URL'] = self.wrong_auth_url
        os.environ['OS_PROJECT_NAME'] = self.wrong_project

        env_vars_dict = dict(
            create_parser().parse_args()._get_kwargs()
        )
        remove_empty(env_vars_dict)
        env_vars_dict.update(kwargs)
        variable = pipeline.get_variables_for_package(PACKAGES.TIONIX_CC,
                                                      **env_vars_dict)

        self.assertEqual(getattr(variable, 'AUTH_PASSWORD'), self.right_pass)
        self.assertEqual(getattr(variable, 'PROJECT_NAME'), self.right_project)
        self.assertEqual(getattr(variable, 'AUTH_URL'), self.right_auth_url)

    def test_get_variables_for_package_with_config_file(self):
        config_file_data = (
            'export OS_AUTH_URL={0}\n'
            'export OS_PROJECT_NAME={1}\n'
            'export OS_PASSWORD={2}'.format(
                self.right_auth_url, self.right_project, self.right_pass))
        self.config_file_fh.write(config_file_data)
        self.config_file_fh.flush()

        sys.argv.append('--OS_AUTH_URL')
        sys.argv.append(self.wrong_auth_url)
        sys.argv.append('--OS_PROJECT_NAME')
        sys.argv.append(self.wrong_project)

        os.environ['OS_AUTH_URL'] = self.wrong_auth_url
        os.environ['OS_PASSWORD'] = self.wrong_pass

        kwargs = {'config_file': self.config_file_fh.name}

        variable = pipeline.get_variables_for_package(PACKAGES.TIONIX_CC,
                                                      **kwargs)

        self.assertEqual(getattr(variable, 'AUTH_URL'), self.right_auth_url)
        self.assertEqual(getattr(variable, 'PROJECT_NAME'), self.right_project)
        self.assertEqual(getattr(variable, 'AUTH_PASSWORD'), self.right_pass)

    def test_get_variables_for_package_with_cc_config_file(self):
        cc_config_file_data = (
            'export TNX_CC_OS_PASSWORD={0}'.format(self.right_tnx_cc_pass))

        self.cc_config_file_fh.write(cc_config_file_data)
        self.cc_config_file_fh.flush()

        os.environ['TNX_CC_OS_PASSWORD'] = self.wrong_tnx_cc_pass

        kwargs = {'config_file': self.cc_config_file_fh.name}

        variable = pipeline.get_variables_for_package(PACKAGES.TIONIX_CC,
                                                      **kwargs)
        self.assertEqual(getattr(variable, 'AUTH_PASSWORD'),
                         self.right_tnx_cc_pass)


class RestoreKeystoneConfigTestCase(unittest.TestCase):
    def setUp(self):
        self.variable_box = DefaultsVariableBox()
        self.variable_box.kwargs.update(
            cloud_domain_policy=True)
        self.temp_dir = tempfile.mkdtemp()
        self.keystone_config_path = os.path.join(
            self.temp_dir, keystone_policy_cfg)

        self.create_config(self.keystone_config_path, is_cloud=True)
        self.generate_sample_backups()

    def create_config(self, file_name, is_cloud=True):
        non_cloud_sample = {
            "admin_required": "role:admin or is_admin:1",
            "identity:get_domain_config_default": "rule:admin_required",
        }

        cloud_sample = {
            "admin_required": "role:admin",
            "cloud_admin": "role:admin and domain_id:123",
            "identity:get_domain_config_default": "rule:cloud_admin",
        }

        data = cloud_sample if is_cloud else non_cloud_sample

        file_path = os.path.join(self.temp_dir, file_name)
        with open(file_path, 'w') as fp:
            json.dump(data, fp)

    def generate_sample_backups(self):
        date_now = datetime.now()
        self.backup_names = []
        backup_num = 5
        for delta_week in range(backup_num):
            backup_date = (date_now - timedelta(weeks=delta_week))
            backup_date_str = backup_date.strftime(rename_date_format)
            backup_name = keystone_policy_cfg + '.' + backup_date_str
            self.backup_names.append(backup_name)
            if len(self.backup_names) < backup_num:
                self.create_config(backup_name)

        self.create_config(self.backup_names[-1], is_cloud=False)

    def tearDown(self):
        shutil.rmtree(self.temp_dir)

    def test_backups_finder(self):
        executor = RestoreKeystoneConfig(self.variable_box)
        executor.config_dir = self.temp_dir
        found_backup = executor._get_noncloud_backup()
        self.assertEqual(self.backup_names[-1], os.path.basename(found_backup))

    def test_precheck(self):
        self.variable_box.kwargs.update(
            cloud_domain_policy=False)
        executor = RestoreKeystoneConfig(self.variable_box)
        self.assertFalse(executor.pre_check())

        self.variable_box.kwargs.update(
            cloud_domain_policy=True)
        executor = RestoreKeystoneConfig(self.variable_box)
        self.assertTrue(executor.pre_check())

    def test_restore_policy(self):
        executor = RestoreKeystoneConfig(self.variable_box)
        executor.config_dir = self.temp_dir
        executor._keystone_cfg_path = self.keystone_config_path

        self.assertTrue(executor._is_cloud_config(self.keystone_config_path))
        executor.run()
        self.assertFalse(executor._is_cloud_config(self.keystone_config_path))

    def test_restore_policy_no_noncloud_backup(self):
        executor = RestoreKeystoneConfig(self.variable_box)
        executor.config_dir = self.temp_dir
        executor._keystone_cfg_path = self.keystone_config_path
        os.remove(os.path.join(self.temp_dir, self.backup_names[-1]))

        self.assertTrue(executor._is_cloud_config(self.keystone_config_path))
        result = executor.run()
        self.assertTrue(executor._is_cloud_config(self.keystone_config_path))
        self.assertTrue(result.failed())

    def test_restore_policy_no_valid_backup(self):
        executor = RestoreKeystoneConfig(self.variable_box)
        executor.config_dir = self.temp_dir
        executor._keystone_cfg_path = self.keystone_config_path

        for file_name in self.backup_names:
            with open(os.path.join(self.temp_dir, file_name), 'w') as fp:
                fp.write('invalid data')

        self.assertIsNone(executor._get_noncloud_backup())
        self.assertTrue(executor._is_cloud_config(self.keystone_config_path))
        result = executor.run()
        self.assertTrue(executor._is_cloud_config(self.keystone_config_path))
        self.assertTrue(result.failed())


class ActualizeConfigTestCase(unittest.TestCase):
    yaml_suffix = '.yaml'
    example_suffix = '.example'

    yaml_orig = {
        'NOVA_VERSION': 2,
        'CINDER_VERSION': 2,
        'KEYSTONE': {
            'auth_url': 'http://controller:5000/v3',
            'auth_version': '3',
            'auth_user': 'admin',
        }
    }

    yaml_example = {
        'NOVA_VERSION': 3,
        'LOG_LEVEL': 'DEBUG',
        'CINDER_VERSION': 2,
        'KEYSTONE': {
            'auth_url': 'http://controller:5000/v3',
            'auth_version': '3',
            'auth_user': 'test',
            'compute_service_name': 'compute',
        }
    }

    expected_yaml = {
        'NOVA_VERSION': 2,
        'LOG_LEVEL': 'DEBUG',
        'CINDER_VERSION': 2,
        'KEYSTONE': {
            'auth_url': 'http://controller:5000/v3',
            'auth_version': '3',
            'auth_user': 'admin',
            'compute_service_name': 'compute',
        }
    }

    def setUp(self):
        _id, self.yaml_file = tempfile.mkstemp(self.yaml_suffix)
        self.yaml_example_file = self.yaml_file + self.example_suffix

        with open(self.yaml_file, 'w') as fp:
            yaml.dump(self.yaml_orig, fp)
        with open(self.yaml_example_file, 'w') as fp:
            yaml.dump(self.yaml_example, fp)

        self.addCleanup(lambda: os.remove(self.yaml_file))
        self.addCleanup(lambda: os.remove(self.yaml_example_file))

    def test_config_update(self):
        actualize_config = ActualizeConfig()
        actualize_config.config_dir, actualize_config.config_name = (
            os.path.split(self.yaml_file))

        actualize_config.run()
        with open(self.yaml_file, 'r') as fp:
            result_yaml = yaml.load(fp)

        self.assertDictEqual(result_yaml, self.expected_yaml)

    @mock.patch.object(ActualizeConfig, '_run')
    def test_config_is_missing(self, mocked_run):
        actualize_config = ActualizeConfig()
        actualize_config.config_name = 'not_exist'

        actualize_config.run()

        self.assertFalse(mocked_run.called)
