# coding: utf-8

import collections
import os
import unittest
from tempfile import NamedTemporaryFile

import yaml
from jinja2 import FileSystemLoader
from jinja2.environment import Environment

from tionix_autosetup.executors.config import write_config_file


# pylint: disable=no-member
class TestTestsCase(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        start = os.path.dirname(__file__)
        templates_dir_path = os.path.join(start, '../templates')
        cls.mode = 'w'
        cls.env = Environment(loader=FileSystemLoader(templates_dir_path))

    def setUp(self):
        self.temp_file = NamedTemporaryFile(delete=False)
        self.config = {}

    def tearDown(self):
        self.temp_file.delete = True
        self.temp_file.close()

    test_dict = {
        'DB_ENGINE': 'mysql',
        'engine': 'mysql',
        'DB_HOST': 'controller',
        'host': 'controller',
        'DB_PORT': '3306',
        'port': '3306',
        'DB_USERNAME': 'tionix',
        'user': 'tionix',
        'DB_USER_PASSWORD': 'password',
        'password': 'password',
        'DB_NAME': 'Test',

        'auth_version': '3',
        'COMPUTE_SERVICE_NAME': 'computev2',
        'CEILOMETER_SERVICE_NAME': 'metering',
        'cinder_version': '2',
        'AUTH_URL': 'http://localhost:5000',
        'auth_url': 'http://localhost:5000',
        'user_domain_name': 'default',
        'NETWORK_API_VERSION': '2',
        'NETWORK_SERVICE_NAME': 'network',
        'IDENTITY_SERVICE_NAME': 'identity',
        'IDENTITY_API_VERSION': '3',
        'volume_service_name': 'volumev2',
        'AUTH_PASSWORD': '123456',
        'PROJECT_NAME': 'admin',
        'auth_tenant': 'admin',
        'VOLUME_SERVICE_NAME': 'volumev2',
        'project_domain_name': 'default',
        'compute_service_name': 'computev2',
        'USER_DOMAIN_NAME': 'default',
        'AUTH_USERNAME': 'admin',
        'PROJECT_DOMAIN_NAME': 'default',
        'VOLUME_API_VERSION': '2',
        'auth_password': '123456',
        'auth_user': 'admin',

        'RABBIT_PORT': '5672',
        'broker_type': 'amqp',
        'rabbit_host': 'controller',
        'RABBIT_USERNAME': 'tionix',
        'rabbit_user': 'tionix',
        'RABBIT_PASSWORD': 'password',
        'rabbit_port': '5672',
        'RABBIT_VIRTHOST': 'tionix_vhost',
        'RABBIT_HOST': 'controller',
        'rabbit_pwd': 'password',
        'BROKER_TYPE': 'amqp',

        'SCHEDULER_HOST': 'localhost',

        'LOG_LEVEL': 'DEBUG',
        'verbose': False,

        'MODULE_USER_DOMAIN_NAME': 'default',
        'MODULE_RABBIT_PASSWORD': 'password',
        'MODULE_DB_PORT': '3306',
        'MODULE_IDENTITY_API_VERSION': '3',
        'MODULE_RABBIT_PORT': '5672',
        'MODULE_PROJECT_NAME': 'admin',
        'MODULE_AUTH_URL': 'http://localhost:5000',
        'MODULE_VOLUME_API_VERSION': '2',
        'MODULE_NETWORK_API_VERSION': '2',
        'MODULE_DB_HOST': 'controller',
        'MODULE_VOLUME_SERVICE_NAME': 'volumev2',
        'MODULE_CEILOMETER_SERVICE_NAME': 'metering',
        'MODULE_NETWORK_SERVICE_NAME': 'network',
        'MODULE_IDENTITY_SERVICE_NAME': 'identity',
        'MODULE_AUTH_PASSWORD': '123456',
        'MODULE_LOG_LEVEL': 'DEBUG',
        'MODULE_DB_USER_PASSWORD': 'password',
        'MODULE_RABBIT_HOST': 'controller',
        'MODULE_AUTH_USERNAME': 'admin',
        'MODULE_DB_USERNAME': 'tionix',
        'MODULE_BROKER_TYPE': 'amqp',
        'MODULE_PROJECT_DOMAIN_NAME': 'default',
        'MODULE_COMPUTE_SERVICE_NAME': 'computev2',
        'MODULE_RABBIT_USERNAME': 'tionix',
        'MODULE_DB_ENGINE': 'mysql'
    }

    def assertConfigEqual(self, dict1, dict2):
        def process_dict(_dict):
            result_dict = collections.OrderedDict()
            result_dict.update(sorted(_dict.items()))
            for key, value in result_dict.items():
                if isinstance(value, dict):
                    result_dict[key] = collections.OrderedDict()
                    result_dict[key].update(sorted(value.items()))
                else:
                    result_dict[key] = str(value)
            return result_dict
        self.assertDictEqual(process_dict(dict1), process_dict(dict2))

    def test_base_config(self):
        ethalon_dict = {
            'CINDER_VERSION': '2',
            'LOG_LEVEL': 'DEBUG',
            'KEYSTONE': {
                'compute_service_name': 'computev2',
                'user_domain_name': 'default',
                'auth_tenant': 'admin',
                'auth_version': '3',
                'auth_url': 'http://localhost:5000',
                'volume_service_name': 'volumev2',
                'ceilometer_service_name': 'metering',
                'auth_user': 'admin',
                'auth_password': '123456',
                'project_domain_name': 'default'
            },
            'DB': {
                'ENGINE': 'mysql',
                'HOST': 'controller',
                'PASSWORD': 'password',
                'USER': 'tionix',
                'PORT': '3306'
            },
            'RABBIT_QUEUES': {
                'username': 'tionix',
                'host': 'controller',
                'password': 'password',
                'broker_type': 'amqp',
                'port': '5672'
            }
        }

        template_name = 'base_config.template'
        template = self.env.get_template(template_name)

        output_line = template.render(self.test_dict)
        write_config_file(self.mode, output_line, self.temp_file.name)

        tmp_config = yaml.load(open(self.temp_file.name))
        self.config.update(tmp_config)

        self.assertConfigEqual(self.config, ethalon_dict)

    def test_base_modules_config(self):
        ethalon_dict = {
            'CINDER_VERSION': '2',
            'LOG_LEVEL': 'DEBUG',
            'KEYSTONE': {
                'compute_service_name': 'computev2',
                'user_domain_name': 'default',
                'auth_tenant': 'admin',
                'auth_version': '3',
                'auth_url': 'http://localhost:5000',
                'volume_service_name': 'volumev2',
                'ceilometer_service_name': 'metering',
                'auth_user': 'admin',
                'auth_password': '123456',
                'project_domain_name': 'default'
            },
            'DB': {
                'ENGINE': 'mysql',
                'HOST': 'controller',
                'PASSWORD': 'password',
                'USER': 'tionix',
                'PORT': '3306'
            },
            'RABBIT_QUEUES': {
                'username': 'tionix',
                'host': 'controller',
                'password': 'password',
                'broker_type': 'amqp',
                'port': '5672'
            }
        }
        self.maxDiff = None
        template_name = 'base_modules_config.template'
        template = self.env.get_template(template_name)

        output_line = template.render(self.test_dict)
        write_config_file(self.mode, output_line, self.temp_file.name)

        tmp_config = yaml.load(open(self.temp_file.name))
        self.config.update(tmp_config)

        self.assertConfigEqual(self.config, ethalon_dict)

    def test_cloud_control_config(self):
        ethalon_dict = {
            'CINDER_VERSION': '2',
            'LOG_LEVEL': 'DEBUG',
            'KEYSTONE': {
                'compute_service_name': 'computev2',
                'user_domain_name': 'default',
                'auth_tenant': 'admin',
                'auth_version': '3',
                'auth_url': 'http://localhost:5000',
                'volume_service_name': 'volumev2',
                'ceilometer_service_name': 'metering',
                'auth_user': 'admin',
                'auth_password': '123456',
                'project_domain_name': 'default'
            },
            'DB': {
                'ENGINE': 'mysql',
                'HOST': 'controller',
                'NAME': 'Test',
                'PASSWORD': 'password',
                'PORT': '3306',
                'USER': 'tionix'
            },
            'RABBIT_QUEUES': {
                'username': 'tionix',
                'vhost': 'tionix_vhost',
                'host': 'controller',
                'broker_type': 'amqp',
                'password': 'password',
                'port': '5672'
            },
            'RETRIES_WAIT_FOR_VM_STATUS': 60,
            'RETRIES_WAIT_FOR_NODE_STATE': 240,
            'SYNC_NOVA_SERVERS_TIME': 60,
            'DB_CONNECTION_MAX_RETRIES': 2,
            'ALLOW_EVACUATE_HOST': True
        }

        template_name = 'cloud_control_config.template'
        template = self.env.get_template(template_name)

        output_line = template.render(self.test_dict)
        write_config_file(self.mode, output_line, self.temp_file.name)

        tmp_config = yaml.load(open(self.temp_file.name))
        self.config.update(tmp_config)

        self.assertConfigEqual(self.config, ethalon_dict)

    def test_monitor_config(self):
        ethalon_dict = {
            'LOG_LEVEL': 'DEBUG',
            'DB': {
                'ENGINE': 'mysql',
                'HOST': 'controller',
                'PASSWORD': 'password',
                'PORT': '3306',
                'USER': 'tionix'
            },
            'RABBIT_QUEUES': {
                'username': 'tionix',
                'host': 'controller',
                'password': 'password',
                'broker_type': 'amqp',
                'port': '5672'
            },
            'CINDER_VERSION': 2,
            'KEYSTONE': {
                'compute_service_name': 'computev2',
                'user_domain_name': 'default',
                'auth_tenant': 'admin',
                'auth_version': '3',
                'auth_url': 'http://localhost:5000',
                'volume_service_name': 'volumev2',
                'ceilometer_service_name': 'metering',
                'auth_user': 'admin',
                'auth_password': '123456',
                'project_domain_name': 'default'
            },
            'CEILOMETER_METERS': [
                'memory.usage',
                'cpu_util',
                'disk.read.requests.rate',
                'disk.write.requests.rate',
                'disk.read.bytes.rate',
                'disk.write.bytes.rate',
                'disk.latency',
                'disk.iops',
                'network.incoming.bytes.rate',
                'network.outgoing.bytes.rate',
                'network.incoming.packets.rate',
                'network.outgoing.packets.rate',
            ]
        }

        template_name = 'monitor_config.template'
        template = self.env.get_template(template_name)

        output_line = template.render(self.test_dict)
        write_config_file(self.mode, output_line, self.temp_file.name)

        tmp_config = yaml.load(open(self.temp_file.name))
        self.config.update(tmp_config)

        self.assertConfigEqual(self.config, ethalon_dict)

    def test_node_control_config(self):
        ethalon_dict = {
            'CINDER_VERSION': '2',
            'LOG_LEVEL': 'DEBUG',
            'DB': {
                'ENGINE': 'mysql',
                'HOST': 'controller',
                'NAME': 'Test',
                'PASSWORD': 'password',
                'PORT': '3306',
                'USER': 'tionix'
            },
            'RABBIT_QUEUES': {
                'username': 'tionix',
                'host': 'controller',
                'password': 'password',
                'broker_type': 'amqp',
                'port': '5672'
            },
            'KEYSTONE': {
                'compute_service_name': 'computev2',
                'user_domain_name': 'default',
                'auth_tenant': 'admin',
                'auth_version': '3',
                'auth_url': 'http://localhost:5000',
                'volume_service_name': 'volumev2',
                'ceilometer_service_name': 'metering',
                'auth_user': 'admin',
                'auth_password': '123456',
                'project_domain_name': 'default'
            },
            'PXE': {
                'conf_dir': '/var/lib/tftpboot/pxelinux.cfg/'
            },
            'SETTINGS_TRACKER': {
                'mutex': 3
            },
            'DHCP_LEASES_FILEPATHS': [
                '/var/lib/dhcp/dhcpd/state/dhcpd.leases',
                '/var/lib/dhcp/dhcpd.leases'
            ],
            'SYNC_NOVA_NODES_TIME': 60,
            'DB_CONNECTION_MAX_RETRIES': 2,
            'TIMEOUT_RESERV_NODE_UP': 15,
            'MAX_TICK_COUNT': 10,
            'SLEEP_TIME': 30,
            'MAX_DOWN_HOSTS': 0,
            'ALLOW_HOST_AUTO_POWER_OFF': False,
            'HOST_RESTART_TIMEOUT': 600,
        }

        template_name = 'node_control_config.template'
        template = self.env.get_template(template_name)

        output_line = template.render(self.test_dict)
        write_config_file(self.mode, output_line, self.temp_file.name)

        tmp_config = yaml.load(open(self.temp_file.name))
        self.config.update(tmp_config)

        self.assertConfigEqual(self.config, ethalon_dict)

    def test_scheduler_client_config(self):
        ethalon_dict = {
            'LOG_LEVEL': 'DEBUG',
            'PORT': 10001,
            'DB': {
                'ENGINE': 'mysql',
                'HOST': 'controller',
                'PASSWORD': 'password',
                'PORT': '3306',
                'USER': 'tionix'
            },
            'RABBIT_QUEUES': {
                'username': 'tionix',
                'host': 'controller',
                'password': 'password',
                'broker_type': 'amqp',
                'port': '5672'
            },
            'CINDER_VERSION': '2',
            'KEYSTONE': {
                'compute_service_name': 'computev2',
                'user_domain_name': 'default',
                'auth_tenant': 'admin',
                'auth_version': '3',
                'auth_url': 'http://localhost:5000',
                'volume_service_name': 'volumev2',
                'ceilometer_service_name': 'metering',
                'auth_user': 'admin',
                'auth_password': '123456',
                'project_domain_name': 'default'
            },
            'HOST': 'localhost'
        }

        template_name = 'scheduler_client_config.template'
        template = self.env.get_template(template_name)

        output_line = template.render(self.test_dict)
        write_config_file(self.mode, output_line, self.temp_file.name)

        tmp_config = yaml.load(open(self.temp_file.name))
        self.config.update(tmp_config)

        self.assertConfigEqual(self.config, ethalon_dict)

    def test_scheduler_config(self):
        ethalon_dict = {
            'LOG_LEVEL': 'DEBUG',
            'DB': {
                'ENGINE': 'django.db.backends.mysql',
                'HOST': 'controller',
                'NAME': 'Test',
                'PASSWORD': 'password',
                'PORT': '3306',
                'USER': 'tionix'
            },
            'RABBIT_QUEUES': {
                'username': 'tionix',
                'host': 'controller',
                'password': 'password',
                'broker_type': 'amqp',
                'port': '5672',
                'vhost': 'tionix_vhost'
            },
            'CINDER_VERSION': '2',
            'KEYSTONE': {
                'compute_service_name': 'computev2',
                'user_domain_name': 'default',
                'auth_tenant': 'admin',
                'auth_version': '3',
                'auth_url': 'http://localhost:5000',
                'volume_service_name': 'volumev2',
                'ceilometer_service_name': 'metering',
                'auth_user': 'admin',
                'auth_password': '123456',
                'project_domain_name': 'default'
            },
            'ENTRY_GROUPS': ['tionix.tasks', 'tionix_tasks'],
            'CELERYBEAT_SYNC_EVERY': 60,
            'CELERYBEAT_MAX_LOOP_INTERVAL': 30
        }

        template_name = 'scheduler_config.template'
        template = self.env.get_template(template_name)

        output_line = template.render(self.test_dict)
        write_config_file(self.mode, output_line, self.temp_file.name)

        tmp_config = yaml.load(open(self.temp_file.name))
        self.config.update(tmp_config)

        self.assertConfigEqual(self.config, ethalon_dict)

    def test_vdi_server_config(self):
        ethalon_dict = {
            'LOG_LEVEL': 'DEBUG',
            'DB': {
                'ENGINE': 'mysql',
                'HOST': 'controller',
                'NAME': 'Test',
                'PASSWORD': 'password',
                'PORT': '3306',
                'USER': 'tionix'
            },
            'RABBIT_QUEUES': {
                'username': 'tionix',
                'host': 'controller',
                'password': 'password',
                'broker_type': 'amqp',
                'port': '5672'
            },
            'CINDER_VERSION': '2',
            'KEYSTONE': {
                'compute_service_name': 'computev2',
                'user_domain_name': 'default',
                'auth_tenant': 'admin',
                'auth_version': '3',
                'auth_url': 'http://localhost:5000',
                'volume_service_name': 'volumev2',
                'ceilometer_service_name': 'metering',
                'auth_user': 'admin',
                'auth_password': '123456',
                'project_domain_name': 'default'
            },
            'LATENCY': 60,
            'WEBROOT': '/vdi',
            'COMPRESS_OFFLINE': True,
            'DEFAULT_AVAILABILITY_ZONE': 'nova',
            'DEBUG': False
        }

        template_name = 'vdi_server_config.template'
        template = self.env.get_template(template_name)

        output_line = template.render(self.test_dict)
        write_config_file(self.mode, output_line, self.temp_file.name)

        tmp_config = yaml.load(open(self.temp_file.name))
        self.config.update(tmp_config)

        self.assertConfigEqual(self.config, ethalon_dict)

    def test_dashboard_config(self):
        ethalon_dict = {
            'CINDER_VERSION': '2',
            'LOG_LEVEL': 'DEBUG',
            'KEYSTONE': {
                'compute_service_name': 'computev2',
                'user_domain_name': 'default',
                'auth_tenant': 'admin',
                'auth_version': '3',
                'auth_url': 'http://localhost:5000',
                'volume_service_name': 'volumev2',
                'ceilometer_service_name': 'metering',
                'auth_user': 'admin',
                'auth_password': '123456',
                'project_domain_name': 'default',
                'network_service_name': 'network',
                'identity_service_name': 'identity'
            },
            'DB': {
                'ENGINE': 'django.db.backends.mysql',
                'HOST': 'controller',
                'NAME': 'Test',
                'PASSWORD': 'password',
                'PORT': '3306',
                'USER': 'tionix'
            },
            'RABBIT_QUEUES': {
                'username': 'tionix',
                'host': 'controller',
                'password': 'password',
                'broker_type': 'amqp',
                'port': '5672'
            },
            'NEUTRON_VERSION': '2'
        }
        template_name = 'dashboard_config.template'
        template = self.env.get_template(template_name)

        output_line = template.render(self.test_dict)
        write_config_file(self.mode, output_line, self.temp_file.name)

        tmp_config = yaml.load(open(self.temp_file.name))
        self.config.update(tmp_config)

        self.assertConfigEqual(self.config, ethalon_dict)

    def test_scheduler_config_output_no_conn(self):
        """Check scheduler config output if no connection info specified"""
        test_dict = {'SCHEDULER_HOST': 'localhost'}

        template_name = 'scheduler_client_config.template'
        template = self.env.get_template(template_name)
        output_line = template.render(test_dict)

        self.assertEqual(output_line, "\n\nHOST: 'localhost'\nPORT: 10001")
