# coding: utf-8

from __future__ import unicode_literals

import tempfile
import unittest
import os
import sys
import yaml

from tionix_autosetup.main import create_parser, remove_empty
from tionix_autosetup.packages import PACKAGES
from tionix_autosetup.pipeline import get_variables_for_package
from tionix_autosetup.variables import (
    FileVariableBox, SysEnvVariableBox, OSAuthURLVar,
    OSProjectVar, OSAuthUserNameVar, OSAuthPasswordVar, ArgsVariableBox,
    VariableBox, BaseVar, DefaultsVariableBox, BaseComplexVar, MODULE_PREFIX)


class VariableTest(unittest.TestCase):
    class BlankVar(BaseVar):
        name = 'NO_VAR'

    class WrongVar(BaseVar):
        name = 'wrong_var'
        extraction = {1: 1}

    class CustomVar(BaseVar):
        name = 'custom'
        default = 'value1'

    class Custom1Var(BaseVar):
        name = 'custom'
        default = 'value1'

    class Custom2Var(BaseVar):
        name = 'custom'
        default = 'value2'

    class ComplexVar(BaseComplexVar):
        name = 'complex'
        right_value = 'right_value'
        values = {'key1': 'wrong_value', 'key2': right_value,
                  'key3': 'wrong_value'}

        @classmethod
        def get_default(cls):
            return cls.values.get('key2', '')

    def test_variable_validator(self):
        self.assertRaises(ValueError, VariableBox,
                          custom_variables={self.WrongVar.name: self.WrongVar})
        old_name = self.WrongVar.name
        self.WrongVar.name = ''
        self.assertRaises(AssertionError, VariableBox,
                          custom_variables={self.WrongVar.name: self.WrongVar})
        self.WrongVar.name = old_name

    def test_blank_var(self):
        var_dict = VariableBox(**{'no_var': 123}).to_dict()
        self.assertEqual(var_dict, {})

    def test_custom_var(self):
        var = DefaultsVariableBox(
            custom_variables={self.CustomVar.name: self.CustomVar},
            **{'custom': self.CustomVar.default})
        self.assertEqual(self.CustomVar.default, var.to_dict().get('custom'))

    def test_to_dict(self):
        var = DefaultsVariableBox().to_dict()
        self.assertIsInstance(var, dict)

    def test_eq(self):
        var = DefaultsVariableBox(
            custom_variables={self.CustomVar.name: self.CustomVar},
            **{'custom': self.CustomVar.default})
        var1 = DefaultsVariableBox(
            custom_variables={self.Custom1Var.name: self.Custom1Var},
            **{'custom': self.Custom1Var.default})
        var2 = DefaultsVariableBox(
            custom_variables={self.Custom2Var.name: self.Custom2Var},
            **{'custom': self.Custom2Var.default})

        self.assertTrue(var == var1)
        self.assertTrue(var != var2)

    def test_complex_var(self):
        var = DefaultsVariableBox(
            custom_variables={self.ComplexVar.name: self.ComplexVar})
        self.assertEqual(self.ComplexVar.right_value,
                         var.to_dict().get('complex'))


class BaseExtractorTestCase(unittest.TestCase):
    yaml_conf_dict = file_data = env_vars = sysargs_vars = bad_variables = {}
    yaml_tmp_file = tmp_file = ''
    package = PACKAGES.TIONIX

    @staticmethod
    def write_yaml_file(data_dict):
        _id, yaml_tmp_file = tempfile.mkstemp(prefix='tnx_tmp_',
                                              suffix='.yaml')
        with open(yaml_tmp_file, 'w') as fh:
            yaml.dump(data_dict, stream=fh)
        return yaml_tmp_file

    @staticmethod
    def write_dict_to_file(data_dict):
        """ Writes dict object to file, in format to use in system environment
        :param data_dict: dict to vrite
        :return: file path to created file
        """
        _id, tmp_file = tempfile.mkstemp(prefix='tnx_tmp_')
        with open(tmp_file, 'w') as fh:
            fh.writelines('\n'.join(
                ('export ' + k + '=' + v for k, v in data_dict.items())))
        return tmp_file

    def init_sys_argv(self):
        for k, v in self.sysargs_vars.items():
            sys.argv.append('--' + k)
            sys.argv.append(v)

    def init_sys_env(self):
        for k, v in self.env_vars.items():
            os.environ[k] = v

    def assertDict(self, test_dict, expected_dict):

        for k, v in expected_dict.items():
            self.assertEqual(v, test_dict[k], k + ': is not equal')

    @staticmethod
    def get_expected(dct):
        return {
            OSAuthUserNameVar.name: dct['OS_USERNAME'],
            OSAuthPasswordVar.name: dct['OS_PASSWORD'],
            OSProjectVar.name: dct['OS_PROJECT_NAME'],
            OSAuthURLVar.name: dct['OS_AUTH_URL'],
        }

    def setUp(self):
        self.fabric = PACKAGES.variable_box_fabric[self.package]

        super(BaseExtractorTestCase, self).setUp()

        # init sys vars
        self.argv = sys.argv[:]

        # init yaml file vars
        if self.yaml_conf_dict:
            self.yaml_tmp_file = self.write_yaml_file(self.yaml_conf_dict)

        # init file vars
        if self.file_data:
            self.tmp_file = self.write_dict_to_file(self.file_data)
            with open(self.tmp_file, 'a+') as fh:
                fh.writelines(self.bad_variables)

            sys.argv.append('--config_file')
            sys.argv.append(self.tmp_file)
        self.init_sys_argv()
        # init env vars
        self.init_sys_env()
        self.env_vars_dict = dict(
            create_parser().parse_args()._get_kwargs()
        )
        remove_empty(self.env_vars_dict)

    def tearDown(self):
        super(BaseExtractorTestCase, self).tearDown()
        # yaml file vars
        if self.yaml_tmp_file:
            os.remove(self.yaml_tmp_file)
        # file vars
        if self.tmp_file:
            os.remove(self.tmp_file)
        # sys vars
        sys.argv = self.argv
        # env vars
        for k, _v in self.env_vars.items():
            if k in os.environ:
                os.environ.pop(k)


class TestFileExtractor(BaseExtractorTestCase):
    file_data = {
        'OS_PROJECT_NAME': 'file_admin',
        'OS_USERNAME': 'file_admin',
        'OS_PASSWORD': 'file_qwe',
        'OS_AUTH_URL': 'file_http://controller:35357/v3',
    }

    bad_variables = '''
    export TEMP_VARIABLE=bad_variable
    export OS_VARIABLE==bad_variable
    export =
    export OS_VARIABLEbad_variable
    exportOS_VARIABLE=bad_variable
    '''

    def test_file_extractor(self):
        from_file_dict = self.fabric(
            FileVariableBox, config_file=self.tmp_file).to_dict()
        expected = self.get_expected(self.file_data)

        self.assertDict(from_file_dict, expected)

    def test_file_extractor_kwargs(self):
        from_file_dict = self.fabric(
            FileVariableBox, **{'config_file': self.tmp_file}).to_dict()
        expected = self.get_expected(self.file_data)

        self.assertDict(from_file_dict, expected)


class TestSysEnvExtractor(BaseExtractorTestCase):
    env_vars = {
        'OS_USERNAME': 'sysenv_admin',
        'OS_PASSWORD': 'sysenv_qwe',
        'OS_PROJECT_NAME': 'sysenv_admin',
        'OS_AUTH_URL': 'sysenv_http://controller:35357/v3',
    }

    def test_sysenv_extractor(self):
        sys_dict = self.fabric(
            SysEnvVariableBox).to_dict()
        expected = self.get_expected(self.env_vars)

        self.assertDict(sys_dict, expected)


class TestSysEnvCollectionExtractor(BaseExtractorTestCase):
    env_vars = {
        'DB_ALLOWED_HOSTS': 'localhost,%',
        'OS_PASSWORD': 'sysenv,_qwe',
    }

    def test_sysenv_collection_extractor(self):
        sys_dict = self.fabric(
            SysEnvVariableBox).to_dict()

        self.assertIsInstance(sys_dict['AUTH_PASSWORD'], basestring)
        self.assertEqual(sys_dict['AUTH_PASSWORD'],
                         self.env_vars['OS_PASSWORD'])
        self.assertIsInstance(sys_dict['DB_ALLOWED_HOSTS'], list)
        self.assertListEqual(sys_dict['DB_ALLOWED_HOSTS'],
                             self.env_vars['DB_ALLOWED_HOSTS'].split(','))


class TestSysArgsExtractor(BaseExtractorTestCase):
    sysargs_vars = {
        'OS_USERNAME': 'sysargs_admin',
        'OS_PASSWORD': 'sysargs_qwe',
        'OS_PROJECT_NAME': 'sysargs_admin',
        'OS_AUTH_URL': 'sysargs_http://controller:35357/v3',
    }

    def get_sysargs_vars(self):
        return self.sysargs_vars

    def test_sys_arg_by_os_variables(self):
        _dict = get_variables_for_package(
            self.package, **self.env_vars_dict).to_dict()
        expected = self.get_expected(self.sysargs_vars)
        self.assertDict(_dict, expected)

    def test_sys_arg_by_kwargs_variables(self):
        _dict = self.fabric(
            ArgsVariableBox, **self.get_sysargs_vars()).to_dict()
        expected = self.get_expected(self.sysargs_vars)
        self.assertDict(_dict, expected)


class ModuleMixin(object):
    PREFIX = ''
    package = PACKAGES.TIONIX

    def add_prefix(self, key, is_module=False):
        if is_module:
            return MODULE_PREFIX + key
        return self.PREFIX + key

    def write_yaml_file(self, data_dict):
        patched_dict = {self.add_prefix(k): self.add_prefix(v)
                        for k, v in data_dict.items()}
        return super(ModuleMixin, self).write_yaml_file(patched_dict)

    def write_dict_to_file(self, data_dict):
        patched_dict = {self.add_prefix(k): self.add_prefix(v)
                        for k, v in data_dict.items()}
        return super(ModuleMixin, self).write_dict_to_file(patched_dict)

    def get_sysargs_vars(self):
        sysargs_vars = getattr(self, 'sysargs_vars', {})
        return {self.add_prefix(k): self.add_prefix(v)
                for k, v in sysargs_vars.items()}

    def get_expected(self, dct):
        return {
            self.add_prefix(OSAuthUserNameVar.name, True): self.add_prefix(
                dct['OS_USERNAME']),
            self.add_prefix(OSAuthPasswordVar.name, True): self.add_prefix(
                dct['OS_PASSWORD']),
            self.add_prefix(OSProjectVar.name, True): self.add_prefix(
                dct['OS_PROJECT_NAME']),
            self.add_prefix(OSAuthURLVar.name, True): self.add_prefix(
                dct['OS_AUTH_URL']),
        }

    def init_sys_argv(self):
        sysargs_vars = getattr(self, 'sysargs_vars', {})
        for k, v in sysargs_vars.items():
            sys.argv.append('--' + self.add_prefix(k))
            sys.argv.append(self.add_prefix(v))

    def init_sys_env(self):
        env_vars = getattr(self, 'env_vars', {})
        for k, v in env_vars.items():
            os.environ[self.add_prefix(k)] = self.add_prefix(v)

    def tearDown(self):
        super(ModuleMixin, self).tearDown()

        # env vars
        env_vars = getattr(self, 'env_vars', {})
        for k, _v in env_vars.items():
            key = self.add_prefix(k)
            if key in os.environ:
                os.environ.pop(key)


class CCModuleMixin(ModuleMixin):
    PREFIX = 'TNX_CC_'
    package = PACKAGES.TIONIX_CC


class TestCCSysArgVariablesByModulePrefix(CCModuleMixin, TestSysArgsExtractor):
    pass


class TestCCSysEnvVariablesByModulePrefix(CCModuleMixin, TestSysEnvExtractor):
    pass


class TestCCFileVariablesByModulePrefix(CCModuleMixin, TestFileExtractor):
    pass


class NCModuleMixin(ModuleMixin):
    PREFIX = 'TNX_NC_'
    package = PACKAGES.TIONIX_NC


class TestNCSysArgVariablesByModulePrefix(NCModuleMixin, TestSysArgsExtractor):
    pass


class TestNCSysEnvVariablesByModulePrefix(NCModuleMixin, TestSysEnvExtractor):
    pass


class TestNCFileVariablesByModulePrefix(NCModuleMixin, TestFileExtractor):
    pass
