# coding: utf-8

from __future__ import unicode_literals

import glob
import json
import locale
import logging
import os
import shutil

from datetime import datetime

from tionix_autosetup.executors.config import (
    DropConfig, RenameConfig, ConfigWithSectionsParser, UpdateSudoers)
from tionix_autosetup.executors.package import Install, Uninstall
from tionix_autosetup.executors.rabbit import DropRabbitUser
from tionix_autosetup.executors.rollback import RollbackExecutor
from tionix_autosetup.executors.base import (
    Executor, AllModulesRemovedMixin, AdminRoleCheckMixin,
    CloudDomainPolicyMixin, AppendPath, Dirs, AutosetupException)
from tionix_autosetup.executors.service import (
    SystemServicesRestart, SupervisorStart, AddUser,
    MemcachedRestart, RunConfiguration)
from tionix_autosetup.executors.apache import ApacheRestart
from tionix_autosetup.executors.config import (
    Config, ActualizeConfig, UpdateYamlConfig)
from tionix_autosetup.helpers import (
    installed_tionix_modules, Platform, ManagerTypesEnum, PackageTypesEnum,
    OperationsEnum)


logger = logging.getLogger(__name__)

yaml_config_name = 'tionix.yaml'
keystone_policy_cfg = 'policy.json'
keystone_dir = '/etc/keystone'
rename_date_format = '%d%m%Y_%H%M%S'

PIP_UPGRADE_STRATEGY = '-U --upgrade-strategy only-if-needed %s'

pip_extra_packages = {
    OperationsEnum.INSTALL: [
        'tionix-client',
        'tionix-licensing',
        PIP_UPGRADE_STRATEGY % 'python-subunit',
        PIP_UPGRADE_STRATEGY % 'packaging',
        PIP_UPGRADE_STRATEGY % 'requestsexceptions',
        PIP_UPGRADE_STRATEGY % 'sqlalchemy-migrate',
        PIP_UPGRADE_STRATEGY % 'jsonpath-rw-ext',
    ],
    OperationsEnum.UPDATE: ['tionix-client'],
    OperationsEnum.UNINSTALL: [
        'tionix-licensing',
        'tionix-client',
        'tionix-scheduler-client',
    ],
}

rpm_extra_packages = {
    OperationsEnum.INSTALL: [
        'python-module-tionix_client',
        'python-module-tionix_licensing',
        'python-module-subunit',
        'python-module-packaging',
        'python-module-requestsexceptions',
        'python-module-migrate',
        'python-module-jsonpath-rw-ext',
        'python-module-ua_parser',
        'python-module-user_agents',
    ],
    OperationsEnum.UPDATE: ['python-module-tionix_client'],
    OperationsEnum.UNINSTALL: [
        'python-module-tionix_licensing',
        'python-module-tionix_client',
        'python-module-tionix_scheduler_client',
    ],
}

extra_packages = {
    PackageTypesEnum.PIP: pip_extra_packages,
    PackageTypesEnum.RPM: rpm_extra_packages,
}

system_packages = {
    Platform.UBUNTU: (
        'gcc-4.7', 'python-dev', 'python-pip', 'supervisor', 'python-celery',
    ),
    Platform.ALT_LINUX: (
        'gcc4.7', 'python-dev', 'python-module-pip', 'supervisor',
        'python-module-celery', 'python-module-django-dbbackend-mysql',
    )
}


class BaseConfig(Config):
    config_path = '/etc/tionix/tionix.yaml'
    template_name = 'base_config.template'


class UpdateBaseYamlConfig(UpdateYamlConfig, BaseConfig):
    module_config = False


class TionixActualizeConfig(ActualizeConfig):
    config_name = yaml_config_name


class AssignAdminRole(CloudDomainPolicyMixin, AdminRoleCheckMixin):
    critical_operation = True

    @property
    def commands(self):
        return ('openstack role add --user {DOMAIN_USER} '
                '--domain {ADMIN_DOMAIN_ID} admin',)

    def pre_check(self, *arg, **kwargs):
        result = False
        if self.cloud_domain_policy:
            try:
                exist = self.check_domain_admin_role()
            except AutosetupException as err:
                logger.error(err)
            else:
                if not exist:
                    result = True
        return result

    def post_check(self, *arg, **kwargs):
        result = True
        if self.cloud_domain_policy:
            try:
                exist = self.check_domain_admin_role()
            except AutosetupException as err:
                logger.error(err)
            else:
                if not exist:
                    logger.critical(u'Admin role is not assigned to user')
                    result = False
        return result


class BackupKeystoneConfig(CloudDomainPolicyMixin, Executor):
    critical_operation = True
    config_name = keystone_policy_cfg
    config_dir = keystone_dir
    renaming_postfix = '.' + datetime.now().strftime(rename_date_format)
    _keystone_cfg_path = os.path.join(keystone_dir, keystone_policy_cfg)
    _keystone_cfg_path_renamed = _keystone_cfg_path + renaming_postfix

    commands = (
        'cp {0} {1}'.format(_keystone_cfg_path, _keystone_cfg_path_renamed),
    )


class RestoreKeystoneConfig(CloudDomainPolicyMixin, Executor):
    critical_operation = True
    config_name = keystone_policy_cfg
    config_dir = keystone_dir
    _keystone_cfg_path = os.path.join(keystone_dir, keystone_policy_cfg)

    def _get_noncloud_backup(self):
        """Get valid backup without cloud admin policy created by Autosetup"""

        pattern = os.path.join(self.config_dir, self.config_name + '.*')
        parsed_backups = {}

        for backup_file in glob.glob(pattern):
            try:
                if not self._is_cloud_config(backup_file):
                    date_str = backup_file.split(self.config_name + '.')[-1]
                    parsed_backups[backup_file] = datetime.strptime(
                        date_str, rename_date_format)
            except (OSError, ValueError):
                continue

        result = sorted(parsed_backups, key=parsed_backups.get)

        return unicode(result[-1]) if result else None

    def post_check(self, *arg, **kwargs):
        return not self._is_cloud_config(self._keystone_cfg_path)

    @staticmethod
    def _is_cloud_config(config_name):
        with open(config_name) as fp:
            data = json.load(fp)

        return "cloud_admin" in data

    def _run(self, **kwargs):
        found_backup = self._get_noncloud_backup()
        if found_backup:
            shutil.copy(found_backup, self._keystone_cfg_path)


class CreateKeystoneConfig(CloudDomainPolicyMixin, Config):
    critical_operation = True
    config_path = os.path.join(keystone_dir, keystone_policy_cfg)
    template_name = 'policy_v3_cloud_sample.template'
    rewrite = True


class HardlinkKeystoneConfig(CloudDomainPolicyMixin, Executor):
    critical_operation = True
    _keystone_policy_cfg_horizon = 'keystone_policy.v3cloudsample.json'

    _keystone_cfg_path = os.path.join(keystone_dir, keystone_policy_cfg)

    def __init__(self, variable_box=None, **kwargs):
        super(HardlinkKeystoneConfig, self).__init__(variable_box, **kwargs)

        horizon_paths = {
            Platform.UBUNTU: '/usr/share/openstack-dashboard/'
                             'openstack_dashboard/conf',
            Platform.ALT_LINUX: '/etc/openstack-dashboard',
        }

        horizon_conf_dir = horizon_paths.get(Platform.distribution)

        keystone_horizon_cfg_path = os.path.join(
            horizon_conf_dir, self._keystone_policy_cfg_horizon)

        self.commands = (
            'ln -f {0} {1}'.format(self._keystone_cfg_path,
                                   keystone_horizon_cfg_path),
        )


class UpdateLocaleVariable(Executor):

    def _run(self, **kwargs):
        lc_all = os.getenv('LC_ALL')
        if not lc_all:
            lang_code, encoding = locale.getdefaultlocale()
            lang = '{0}.{1}'.format(lang_code, encoding)
            os.environ['LC_ALL'] = os.getenv('LANG') or lang


class UpdateNovaVersionConf(ConfigWithSectionsParser):
    config_path = '/etc/nova/nova.conf'
    config_section = 'osapi_v3'
    config_keys = {
        'enabled': 'True',
    }


class TionixDirs(Dirs):
    dirs = [
        '/etc/tionix/',
        '/var/log/tionix/',
    ]


class RunSelfDiagnosticTest(Executor):
    result_log_level = logging.INFO

    commands = (
        'openstack tnx tests',
    )

    def pre_check(self, *arg, **kwargs):
        return installed_tionix_modules()


class TionixYamlRename(RenameConfig):
    config_name = yaml_config_name

    @property
    def paths(self):
        config_paths = []
        if not installed_tionix_modules():
            config_paths = super(TionixYamlRename, self).paths
        return config_paths


class TionixYamlDrop(DropConfig):
    config_name = yaml_config_name

    @property
    def paths(self):
        config_paths = []
        if not installed_tionix_modules():
            config_paths = super(TionixYamlDrop, self).paths
        return config_paths


class InstallExtraTionix(Install):
    critical_operation = False
    package_names = extra_packages


class UninstallExtraTionix(AllModulesRemovedMixin, Uninstall):
    package_names = extra_packages


class CommonSystemPackages(Install):
    default_manager = ManagerTypesEnum.APT
    packages = system_packages.get(Platform.distribution)


class ClientConfiguration(RunConfiguration):
    module_names = 'tnx_client'


class AptGetUpdate(Executor):
    commands = (
        'apt-get update',
    )


pipeline = [
    AptGetUpdate,
    InstallExtraTionix,
    ClientConfiguration,
    AppendPath,
    AddUser,
    UpdateSudoers,
    TionixDirs,
    UpdateNovaVersionConf,
    BaseConfig,
    UpdateBaseYamlConfig,
    TionixActualizeConfig,
    UpdateLocaleVariable,
    CommonSystemPackages,
    SupervisorStart,
    AssignAdminRole,
    BackupKeystoneConfig,
    CreateKeystoneConfig,
    HardlinkKeystoneConfig,
]

update_pipeline = [
    ClientConfiguration,
    UpdateBaseYamlConfig,
    TionixActualizeConfig,
    AppendPath,
    AssignAdminRole,
    BackupKeystoneConfig,
    CreateKeystoneConfig,
    HardlinkKeystoneConfig,
]

connections_check_pipeline = [AppendPath]

post_check_pipeline = [
    RunSelfDiagnosticTest,
]

rollback_pipeline = [
    RollbackExecutor,
    ApacheRestart,
    MemcachedRestart,
    SystemServicesRestart,
]

uninstall_pipeline = [
    UninstallExtraTionix,
    TionixYamlRename,
    TionixYamlDrop,
    AppendPath,
    DropRabbitUser,
    BackupKeystoneConfig,
    RestoreKeystoneConfig,
    ApacheRestart,
    MemcachedRestart,
    SystemServicesRestart,
]
