# coding: utf-8

from __future__ import unicode_literals

import logging
import os
import re

from tionix_autosetup.executors.base import (
    Executor, CloudDomainPolicyMixin, MODULES_CONF_DIR)
from tionix_autosetup.executors.config import (
    ConfigFileParser, Config, UpdateYamlConfig, ActualizeConfig, DropConfig)
from tionix_autosetup.executors.mysql import (
    DbConnectionCheck, MysqlDropCommands, MysqlCommands)
from tionix_autosetup.executors.keystone import KeystoneConnectionCheck
from tionix_autosetup.executors.package import (
    Install, Update, Uninstall)
from tionix_autosetup.executors.service import (
    SystemServicesRestart, Migration, MemcachedRestart,
    RunConfiguration)
from tionix_autosetup.executors.apache import ApacheRestart
from tionix_autosetup.helpers import (
    Platform, ManagerTypesEnum, PackageTypesEnum, OperationsEnum)


logger = logging.getLogger(__name__)

yaml_config_name = 'dashboard.yaml'


package_names = {
    PackageTypesEnum.PIP: 'tionix-dashboard',
    PackageTypesEnum.RPM: 'python-module-tionix_dashboard',
}


extra_packages = {
    PackageTypesEnum.PIP: {
        OperationsEnum.INSTALL: [
            'tionix-dashboard-theme',
            '-U --upgrade-strategy only-if-needed django-openstack-auth'
        ],
        OperationsEnum.UPDATE: ['tionix-scheduler-client'],
        OperationsEnum.UNINSTALL: ['tionix-dashboard-theme'],
    },
    PackageTypesEnum.RPM: {
        OperationsEnum.INSTALL: [
            'python-module-tionix_dashboard_theme',
            'python-module-django-openstack-auth'],
        OperationsEnum.UPDATE: ['python-module-tionix_scheduler_client'],
        OperationsEnum.UNINSTALL: ['python-module-tionix_dashboard_theme'],
    }
}


system_packages = {
    Platform.ALT_LINUX: (
        'gettext-tools',
    )
}


class DashboardConfig(Config):
    config_path = os.path.join(MODULES_CONF_DIR, yaml_config_name)
    template_name = 'dashboard_config.template'


class UpdateDashboardYamlConfig(UpdateYamlConfig, DashboardConfig):
    pass


class DashboardDropConfig(DropConfig):
    config_name = yaml_config_name


class DashboardActualizeConfig(ActualizeConfig):
    config_name = yaml_config_name


class ManageExecutor(Executor):
    manage = 'python /usr/share/openstack-dashboard/manage.py '


class DashboardMigration(Migration):
    module_name = 'tnx_dashboard'


class DashboardConfiguration(RunConfiguration):
    module_names = 'tnx_dashboard'


class Static(ManageExecutor):
    commands = (
        ManageExecutor.manage + 'collectstatic --noinput',
        ManageExecutor.manage + 'compress --force',
    )


class CompileMessages(ManageExecutor):
    commands = (
        ManageExecutor.manage + 'compilemessages',
    )


class DashConfig(ConfigFileParser):
    config_names = ('local_settings.py', 'local_settings')
    config_dir = '/etc/openstack-dashboard'
    _config_path = None

    @property
    def config_path(self):
        if not self._config_path:
            for config_name in self.config_names:
                path = os.path.join(self.config_dir, config_name)
                if os.path.exists(path):
                    self._config_path = path
                    break
        return self._config_path


class ImportDashConfig(DashConfig):
    config_string = """
try:
    from tionix_dashboard.settings import *
except ImportError:
    pass
"""

    def setup_new_config_data(self):
        self.new_config_data = {
            self.config_string: self.make_pattern(self.config_string)
        }


class RemoveImportDashConfig(ImportDashConfig):
    delete = True


class CloudPolicyParameters(CloudDomainPolicyMixin, DashConfig):

    policy_files = """
POLICY_FILES = {{
    'identity': 'keystone_policy.v3cloudsample.json',
    'compute': 'nova_policy.json',
    'volume': 'cinder_policy.json',
    'image': 'glance_policy.json',
    'orchestration': 'heat_policy.json',
    'network': 'neutron_policy.json',
    'telemetry': 'ceilometer_policy.json',
}}
"""

    session_engine = """
SESSION_ENGINE = 'django.contrib.sessions.backends.cache'
"""

    caches = """
CACHES = {{
    'default': {{
        'BACKEND': 'django.core.cache.backends.memcached.MemcachedCache',
        'LOCATION': '{MEMCACHE_LOCATION}',
    }},
}}
"""

    multi_domain = '\nOPENSTACK_KEYSTONE_MULTIDOMAIN_SUPPORT = True\n'

    default_domain = """
OPENSTACK_KEYSTONE_DEFAULT_DOMAIN = '{ADMIN_DOMAIN_ID}'
"""
    policy_files_reg = (
        r"\n\s*POLICY_FILES\s*=\s*\{\n\s*('|\")identity('|\"):"
        r"\s*('|\")keystone_policy.v3cloudsample.json('|\"),"
        r"\n\s*('|\")compute('|\"):\s*('|\")nova_policy.json('|\"),"
        r"\n\s*('|\")volume('|\"):\s*('|\")cinder_policy.json('|\"),"
        r"\n\s*('|\")image('|\"):\s*('|\")glance_policy.json('|\"),"
        r"\n\s*('|\")orchestration('|\"):\s*('|\")heat_policy.json('|\"),"
        r"\n\s*('|\")network('|\"):\s*('|\")neutron_policy.json('|\"),"
        r"\n\s*('|\")telemetry('|\"):\s*('|\")ceilometer_policy.json('|\"),"
        r"\n\s*\}")

    caches_reg = (r"\n\s*CACHES\s*=\s*\{\n\s*('|\")default('|\"):"
                  r"\s*\{\n\s*('|\")BACKEND('|\"):\s*('|\")(.*)('|\"),"
                  r"\n\s*('|\")LOCATION('|\"):\s*('|\")(.*)('|\"),\n\s*\},"
                  r"\n\s*\}")

    multi_domain_reg = (r'\n\s*OPENSTACK_KEYSTONE_MULTIDOMAIN_SUPPORT\s*='
                        r'\s*(.*)\n')
    default_domain_reg = (r"\n\s*OPENSTACK_KEYSTONE_DEFAULT_DOMAIN\s*="
                          r"\s*('|\")(.*)('|\")\n")
    session_engine_reg = r"\n\s*SESSION_ENGINE\s*=\s*('|\")(.*)('|\")\n"

    def setup_new_config_data(self):
        self.new_config_data = {
            self.session_engine: re.compile(self.session_engine_reg),
            self.multi_domain: re.compile(self.multi_domain_reg),
            self.default_domain: re.compile(self.default_domain_reg),
            self.policy_files: re.compile(self.policy_files_reg),
            self.caches: re.compile(self.caches_reg),
        }


class UnsetCloudPolicyParameters(CloudDomainPolicyMixin, DashConfig):
    policy_files = """
POLICY_FILES = {{
    'identity': 'keystone_policy.json',
    'compute': 'nova_policy.json',
    'volume': 'cinder_policy.json',
    'image': 'glance_policy.json',
    'orchestration': 'heat_policy.json',
    'network': 'neutron_policy.json',
    'telemetry': 'ceilometer_policy.json',
}}
"""

    policy_files_reg = (
        r"\n\s*POLICY_FILES\s*=\s*\{\n\s*('|\")identity('|\"):"
        r"\s*('|\")keystone_policy.v3cloudsample.json('|\"),"
        r"\n\s*('|\")compute('|\"):\s*('|\")nova_policy.json('|\"),"
        r"\n\s*('|\")volume('|\"):\s*('|\")cinder_policy.json('|\"),"
        r"\n\s*('|\")image('|\"):\s*('|\")glance_policy.json('|\"),"
        r"\n\s*('|\")orchestration('|\"):\s*('|\")heat_policy.json('|\"),"
        r"\n\s*('|\")network('|\"):\s*('|\")neutron_policy.json('|\"),"
        r"\n\s*('|\")telemetry('|\"):\s*('|\")ceilometer_policy.json('|\"),"
        r"\n\s*\}")

    def setup_new_config_data(self):
        self.new_config_data = {
            self.policy_files: re.compile(self.policy_files_reg),
        }


class InstallDash(Install):
    package_names = package_names


class UpdateDash(Update):
    package_names = package_names


class UninstallDash(Uninstall):
    package_names = package_names


class InstallExtraDash(Install):
    critical_operation = False
    package_names = extra_packages


class UpdateExtraDash(Update):
    package_names = extra_packages


class UninstallExtraDash(Uninstall):
    package_names = extra_packages


class DashboardSystemPackages(Install):
    default_manager = ManagerTypesEnum.APT
    packages = system_packages.get(Platform.distribution)


pipeline = [
    InstallDash,
    InstallExtraDash,
    DashboardConfiguration,
    DashboardSystemPackages,
    DashboardConfig,
    UpdateDashboardYamlConfig,
    DashboardActualizeConfig,
    ImportDashConfig,
    MysqlCommands,
    DashboardMigration,
    Static,
    CompileMessages,
    CloudPolicyParameters,
    ApacheRestart,
    MemcachedRestart,
    SystemServicesRestart,
]

update_pipeline = [
    UpdateDash,
    UpdateExtraDash,
    DashboardConfiguration,
    UpdateDashboardYamlConfig,
    DashboardActualizeConfig,
    DashboardMigration,
    Static,
    CloudPolicyParameters,
    ApacheRestart,
    MemcachedRestart,
    SystemServicesRestart,
]

uninstall_pipeline = [
    UninstallDash,
    UninstallExtraDash,
    RemoveImportDashConfig,
    UnsetCloudPolicyParameters,
    MysqlDropCommands,
    DashboardDropConfig,
    Static,
    ApacheRestart,
    MemcachedRestart,
    SystemServicesRestart,
]

connections_check_pipeline = [
    DbConnectionCheck,
    KeystoneConnectionCheck,
]

post_check_pipeline = []
