# coding: utf-8

from __future__ import unicode_literals

import logging
import os

from tionix_autosetup.executors.apache import (
    ApacheRestart, DropApacheConfig, UpdateApacheConfig, ApacheConfigPathMixin)
from tionix_autosetup.executors.base import (
    Executor, AppendPath, MODULES_CONF_DIR, Dirs)
from tionix_autosetup.executors.config import (
    DropPath, DropConfig, RenameConfig, DropSupervisorConfig,
    ConfigWithSectionsParser, Config, UpdateYamlConfig, ActualizeConfig)
from tionix_autosetup.executors.keystone import KeystoneConnectionCheck
from tionix_autosetup.executors.mysql import (
    DbConnectionCheck, MysqlDropCommands, MysqlCommands)
from tionix_autosetup.executors.package import (
    Install, Update, Uninstall)
from tionix_autosetup.executors.rabbit import (
    RabbitMQConnectionCheck, RabbitCommands, DropRabbitVhost)
from tionix_autosetup.executors.service import (
    SystemServicesRestart, Migration, RunConfiguration, SupervisorRestart)
from tionix_autosetup.helpers import PackageTypesEnum, OperationsEnum
from tionix_autosetup.tionix_scheduler.variables import SchedulerHostVar

logger = logging.getLogger(__name__)
dirs = ['/var/log/tionix/scheduler']
config_name = 'scheduler.yaml'
client_config_name = 'scheduler_client.yaml'
module_config_prefix = 'tnx_scheduler'
apache_config_name = 'tionix-scheduler-dash.conf'
tionix_scheduler_beat = 'tionix-scheduler-beat'
tionix_scheduler_worker = 'tionix-scheduler-worker'
supervisor_process = '{0} {1}'.format(
    tionix_scheduler_beat, tionix_scheduler_worker)


package_names = {
    PackageTypesEnum.PIP: 'tionix-scheduler',
    PackageTypesEnum.RPM: 'python-module-tionix_scheduler',
}


extra_packages = {
    PackageTypesEnum.PIP: {
        OperationsEnum.UPDATE: ['tionix-scheduler-client'],
    },
    PackageTypesEnum.RPM: {
        OperationsEnum.UPDATE: ['python-module-tionix_scheduler_client'],
    }
}


class SchedulerMigration(Migration):
    module_name = 'tnx_scheduler'


class SchedulerConfiguration(RunConfiguration):
    module_names = ('tnx_scheduler', 'tnx_scheduler_client')


class CopyApacheConfig(Executor):
    commands = (
        'ln -s /etc/apache2/conf-available/{0} '
        '/etc/apache2/conf-enabled/{0}'.format(apache_config_name),
        'ln -s /etc/httpd2/conf/sites-available/{0} '
        '/etc/httpd2/conf/sites-enabled/{0}'.format(apache_config_name),
        'ln -s /etc/httpd/conf/sites-available/{0} '
        '/etc/httpd/conf/sites-enabled/{0}'.format(apache_config_name),
    )


class SchedulerUpdateApacheConfig(ApacheConfigPathMixin, UpdateApacheConfig):
    config_name = apache_config_name


class SchedulerConfig(Config):
    config_path = os.path.join(MODULES_CONF_DIR, config_name)
    template_name = 'scheduler_config.template'


class SchedulerActualizeConfig(ActualizeConfig):
    config_name = config_name


class SchedulerClientActualizeConfig(ActualizeConfig):
    config_name = client_config_name


class SchedulerClientConfig(Config):
    config_path = os.path.join(MODULES_CONF_DIR, client_config_name)
    template_name = 'scheduler_client_config.template'


class UpdateSchedulerYamlConfig(UpdateYamlConfig):
    pass


class UpdateSchedulerClientYamlConfig(UpdateYamlConfig, SchedulerClientConfig):
    module_varables_names = (SchedulerHostVar.name, )


class UpdateConfigSchedulerBeat(ConfigWithSectionsParser):
    config_path = '/etc/supervisor/conf.d/tnx_scheduler_beat.conf'
    config_section = 'program:{0}'.format(tionix_scheduler_beat)
    config_keys = {
        'user': '{TNX_USERNAME}',
        'group': '{TNX_USERNAME}',
    }


class UpdateConfigSchedulerWorker(ConfigWithSectionsParser):
    config_path = '/etc/supervisor/conf.d/tnx_scheduler_worker.conf'
    config_section = 'program:{0}'.format(tionix_scheduler_worker)
    config_keys = {
        'user': '{TNX_USERNAME}',
        'group': '{TNX_USERNAME}',
    }


class SchedulerDirs(Dirs):
    dirs = dirs


class SchedulerDropConfig(DropConfig):
    config_name = config_name


class SchedulerDropSupervisorConfig(DropSupervisorConfig):
    module_config_prefix = module_config_prefix


class SchedulerRenameConfig(RenameConfig):
    config_name = config_name


class SchedulerDropApacheConfig(DropApacheConfig):
    config_name = 'tionix-scheduler-dash.conf'


class SchedulerDropLogFiles(DropPath):
    paths = dirs


class InstallScheduler(Install):
    package_names = package_names


class UpdateScheduler(Update):
    package_names = package_names


class UninstallScheduler(Uninstall):
    package_names = package_names


class UpdateExtraScheduler(Update):
    package_names = extra_packages


class SupervisorRestartScheduler(SupervisorRestart):

    @property
    def commands(self):
        return [self.commands_template.format(supervisor_process)]

pipeline = [
    InstallScheduler,
    SchedulerConfiguration,
    SchedulerDirs,
    SchedulerConfig,
    UpdateSchedulerYamlConfig,
    SchedulerClientConfig,
    UpdateSchedulerClientYamlConfig,
    SchedulerActualizeConfig,
    SchedulerClientActualizeConfig,
    RabbitCommands,
    MysqlCommands,
    CopyApacheConfig,
    SchedulerUpdateApacheConfig,
    UpdateConfigSchedulerBeat,
    UpdateConfigSchedulerWorker,
    SchedulerMigration,
    ApacheRestart,
    SystemServicesRestart,
]

update_pipeline = [
    UpdateScheduler,
    UpdateExtraScheduler,
    SchedulerConfiguration,
    UpdateSchedulerYamlConfig,
    UpdateSchedulerClientYamlConfig,
    SchedulerActualizeConfig,
    SchedulerClientActualizeConfig,
    SchedulerMigration,
    ApacheRestart,
    SupervisorRestartScheduler,
    SystemServicesRestart,
]

uninstall_pipeline = [
    UninstallScheduler,
    AppendPath,
    SchedulerRenameConfig,
    MysqlDropCommands,
    SchedulerDropConfig,
    SchedulerDropSupervisorConfig,
    SchedulerDropApacheConfig,
    SchedulerDropLogFiles,
    DropRabbitVhost,
    ApacheRestart,
    SystemServicesRestart,
]

connections_check_pipeline = [
    DbConnectionCheck,
    KeystoneConnectionCheck,
    RabbitMQConnectionCheck,
]

post_check_pipeline = []
