# coding: utf-8

from __future__ import unicode_literals

from collections import defaultdict

import logging
import os
import yaml
from pkg_resources import DistributionNotFound

from tionix_autosetup.helpers import (
    Versions, get_formatting_distr, classproperty)

logger = logging.getLogger(__name__)


class BaseVar(object):
    # name that is stored
    name = None
    # default value for variable
    default = ''
    # variable is collection
    is_collection = False
    # extraction direction takes first that is found
    extraction = ()


class ModuleBaseVar(BaseVar):
    """Module's specific variable base class"""


class BaseComplexVar(BaseVar):
    """Base complex var when the value changes from conditions"""
    @classmethod
    def get_default(cls):
        raise NotImplementedError()

    # pylint: disable=no-self-argument
    @classproperty
    def default(cls):
        return cls.get_default()


# Typical
class UserNameVar(BaseVar):
    """TIONIX Username"""
    name = 'TNX_USERNAME'
    default = 'tionix'


class LogLevelVar(BaseVar):
    """Variable for level of logging"""
    name = 'LOG_LEVEL'
    default = 'INFO'


# OS section
class OSAuthURLVar(BaseVar):
    """OpenStack Authentication URL"""
    name = 'AUTH_URL'
    default = 'http://localhost:5000'
    extraction = 'OS_AUTH_URL'


class OSAuthUserNameVar(BaseVar):
    """OpenStack Authentication Username"""
    name = 'AUTH_USERNAME'
    default = 'admin'
    extraction = 'OS_USERNAME'


class OSAuthPasswordVar(BaseVar):
    """OpenStack Authentication Password"""
    name = 'AUTH_PASSWORD'
    extraction = 'OS_PASSWORD'


class OSProjectVar(BaseVar):
    """OpenStack Project Name"""
    name = 'PROJECT_NAME'
    default = 'admin'
    extraction = ('OS_PROJECT_NAME', 'OS_TENANT_NAME')


class ComputeServiceNameVar(BaseVar):
    """OpenStack Compute Service Type"""
    name = 'COMPUTE_SERVICE_NAME'
    default = 'compute'


class VolumeServiceNameVar(BaseVar):
    """OpenStack Volume Service Type"""
    name = 'VOLUME_SERVICE_NAME'
    default = 'volumev2'


class VolumeServiceApiVersionVar(BaseVar):
    """OpenStack Volume Service API Version"""
    name = 'VOLUME_API_VERSION'
    default = '2'
    extraction = 'OS_VOLUME_API_VERSION'


class CeilometerServiceNameVar(BaseVar):
    """OpenStack Ceilometer Service Type"""
    name = 'CEILOMETER_SERVICE_NAME'
    default = 'metering'


class NetworkServiceNameVar(BaseVar):
    """OpenStack Network Service Type"""
    name = 'NETWORK_SERVICE_NAME'
    default = 'network'


class IdentityServiceNameVar(BaseVar):
    """OpenStack Identity Service Type"""
    name = 'IDENTITY_SERVICE_NAME'
    default = 'identity'


class NetworkServiceApiVersionVar(BaseVar):
    """OpenStack Network Service API Version"""
    name = 'NETWORK_API_VERSION'
    default = '2'
    extraction = 'OS_NETWORK_API_VERSION'


class IdentityApiVersionVar(BaseVar):
    """OpenStack Identity Service API Version"""
    name = 'IDENTITY_API_VERSION'
    default = '3'
    extraction = 'OS_IDENTITY_API_VERSION'


class ProjectDomainNameVar(BaseVar):
    """Openstack Project Domain"""
    name = 'PROJECT_DOMAIN_NAME'
    default = 'default'
    extraction = 'OS_PROJECT_DOMAIN_NAME'


class UserDomainNameVar(BaseVar):
    """Openstack User Domain"""
    name = 'USER_DOMAIN_NAME'
    default = 'default'
    extraction = 'OS_USER_DOMAIN_NAME'


# DB section
class DBSuperUserVar(BaseVar):
    """User with access to Database"""
    name = 'DB_SUPERUSER_NAME'
    default = 'root'


class DBSuperUserPasswordVar(BaseVar):
    """Database Password"""
    name = 'DB_SUPERUSER_PASSWORD'
    extraction = ('OS_PASSWORD', 'DB_SUPERUSER_PASSWORD')


class DBUsernameVar(BaseVar):
    """Database Username"""
    name = 'DB_USERNAME'
    default = 'tionix'


class DBUserPasswordVar(BaseVar):
    """Database User Password"""
    name = 'DB_USER_PASSWORD'
    default = 'password'
    extraction = ('OS_PASSWORD', 'DB_USER_PASSWORD')


class DBEngineVar(BaseVar):
    """Database Engine"""
    name = 'DB_ENGINE'
    default = 'mysql'


class DBHostVar(BaseVar):
    """Database host"""
    name = 'DB_HOST'
    default = 'controller'


class DBPortVar(BaseVar):
    """Database Port"""
    name = 'DB_PORT'
    default = '3306'


class DBAllowedHostsVar(BaseVar):
    """Hosts that allowed to connect to database.
    Values separated by space using in console arguments
     (example: --DB_ALLOWED_HOSTS localhost controller).
    Values separated by comma using in environment variables
     (example: export DB_ALLOWED_HOSTS=localhost,controller).
    """
    name = 'DB_ALLOWED_HOSTS'
    is_collection = True
    default = ('%',)


class DBNameVar(BaseVar):
    """Database name"""
    name = 'DB_NAME'


# RABBIT section
class RabbitBrokerTypeVar(BaseVar):
    """Rabbit Service Broker Type"""
    name = 'BROKER_TYPE'
    default = 'amqp'


class RabbitHostVar(BaseVar):
    """Rabbit Service Host"""
    name = 'RABBIT_HOST'
    default = 'controller'


class RabbitPortVar(BaseVar):
    """Rabbit Service Port"""
    name = 'RABBIT_PORT'
    default = '5672'


class RabbitUserNameVar(BaseVar):
    """Rabbit Service Username"""
    name = 'RABBIT_USERNAME'
    default = 'tionix'


class RabbitPasswordVar(BaseVar):
    """Rabbit Service Password"""
    name = 'RABBIT_PASSWORD'
    default = 'password'


class RabbitVirtHostVar(BaseVar):
    """Rabbit Service Virtual Host"""
    name = 'RABBIT_VIRTHOST'
    default = 'tionix_vhost'


# other
class PypiHostVar(BaseComplexVar):
    """Tionix pypi host url"""
    name = 'PYPI_HOST'
    values = {Versions.MITAKA: 'pypi.tionix.ru',
              Versions.LIBERTY: 'pypi.tionix.ru'}

    @classmethod
    def get_default(cls):
        try:
            os_version = get_formatting_distr('nova')
        except DistributionNotFound:
            os_version = Versions.MITAKA
        return cls.values.get(os_version, '')


class PipArgsVar(BaseVar):
    """Additional arguments for pip command"""
    name = 'PIP_ARGS'
    default = ''


class AdminDomainID(BaseVar):
    """Admin domain ID"""
    name = 'ADMIN_DOMAIN_ID'


class DomainUser(BaseVar):
    """User with cloud admin role"""
    name = 'DOMAIN_USER'
    default = 'admin'
    extraction = ('OS_USERNAME', 'DOMAIN_USER')


class MemcacheLocation(BaseVar):
    """Url for memory object caching system"""
    name = 'MEMCACHE_LOCATION'
    default = 'controller:11211'


variables = [
    # Typical
    UserNameVar,
    LogLevelVar,

    # DB section
    DBSuperUserVar,
    DBSuperUserPasswordVar,
    DBUsernameVar,
    DBUserPasswordVar,
    DBEngineVar,
    DBHostVar,
    DBPortVar,
    DBNameVar,
    DBAllowedHostsVar,

    # OS section
    OSAuthURLVar,
    OSAuthUserNameVar,
    OSAuthPasswordVar,
    OSProjectVar,
    ProjectDomainNameVar,
    UserDomainNameVar,

    IdentityApiVersionVar,
    IdentityServiceNameVar,

    VolumeServiceApiVersionVar,
    VolumeServiceNameVar,

    CeilometerServiceNameVar,

    NetworkServiceApiVersionVar,
    NetworkServiceNameVar,

    ComputeServiceNameVar,

    # RABBIT section
    RabbitBrokerTypeVar,
    RabbitHostVar,
    RabbitPortVar,
    RabbitUserNameVar,
    RabbitPasswordVar,
    RabbitVirtHostVar,

    # other
    PypiHostVar,
    PipArgsVar,

    # cloud policy
    AdminDomainID,
    DomainUser,
    MemcacheLocation,
]


def get_yaml_variables_matching():
    from tionix_autosetup.tionix_scheduler.variables import SchedulerHostVar

    return {
        'LOG_LEVEL': LogLevelVar.name,
        'NEUTRON_VERSION': NetworkServiceApiVersionVar.name,
        'CINDER_VERSION': VolumeServiceApiVersionVar.name,
        'KEYSTONE': {
            'auth_version': IdentityApiVersionVar.name,
            'compute_service_name': ComputeServiceNameVar.name,
            'network_service_name': NetworkServiceNameVar.name,
            'volume_service_name': VolumeServiceNameVar.name,
            'ceilometer_service_name': CeilometerServiceNameVar.name,
            'identity_service_name': IdentityServiceNameVar.name,
            'auth_url': OSAuthURLVar.name,
            'auth_user': OSAuthUserNameVar.name,
            'auth_password': OSAuthPasswordVar.name,
            'auth_tenant': OSProjectVar.name,
            'user_domain_name': UserDomainNameVar.name,
            'project_domain_name': ProjectDomainNameVar.name,
        },
        'DB': {
            'ENGINE': DBEngineVar.name,
            'NAME': DBNameVar.name,
            'USER': DBUsernameVar.name,
            'PASSWORD': DBUserPasswordVar.name,
            'HOST': DBHostVar.name,
            'PORT': DBPortVar.name,
        },
        'RABBIT_QUEUES': {
            'broker_type': RabbitBrokerTypeVar.name,
            'host': RabbitHostVar.name,
            'port': RabbitPortVar.name,
            'username': RabbitUserNameVar.name,
            'password': RabbitPasswordVar.name,
            'vhost': RabbitVirtHostVar.name,
        },
        'HOST': SchedulerHostVar.name,
    }

MODULE_PREFIX = 'MODULE_'


class VariableBox(object):
    PREFIX = ''

    # variables is collected
    __collected = False

    def __init__(self, variables_prefix=None, custom_variables=None, **kwargs):
        """
        Variable container for extracted variables by extraction rules
        :param variables_prefix: custom module prefix used for extraction
        :type variables_prefix: basestring

        :param custom_variables: extend base variables list
        :type custom_variables: dict
        """
        super(VariableBox, self).__init__()

        # prefix for module custom variable names

        self.PREFIX = variables_prefix or self.PREFIX

        self._variables_dict = defaultdict(lambda: None)

        self.all_variables = variables[:]
        self.collection_variables = [_variable.name
                                     for _variable in self.all_variables
                                     if _variable.is_collection]
        self.kwargs = kwargs

        if custom_variables:
            self.all_variables.extend(custom_variables.values())

        self.all_variables_names = set()

        for var in self.all_variables:
            var_name = var.name
            extraction = var.extraction or var.name

            assert issubclass(var, BaseVar), 'Wrong variable %s' % var
            assert var_name and extraction

            if isinstance(extraction, (tuple, set)):
                self.all_variables_names.update(list(extraction))
            elif isinstance(extraction, (str, unicode)):
                self.all_variables_names.add(extraction)
            else:
                raise ValueError(
                    'Unrecognized global name attribute: {0} '
                    'for variable: {1}'.format(extraction, var))

        self.all_variables_names = set(self.all_variables_names)

        # extend by prefix
        if self.PREFIX:
            self.all_variables_names.update(
                [self.append_prefix(x) for x in self.all_variables_names])

    def _extract(self):
        """ Extract all variables in desired order"""
        self.__collected = True

        extracted_dict = self._get_values()

        result = {}
        for extracted_key in extracted_dict:
            if extracted_key not in self.all_variables_names:
                logger.debug('Skipping variable: {0} = {1}'.format(
                    extracted_key, extracted_dict[extracted_key]))
                continue

            for var in self.all_variables:
                extraction = var.extraction or var.name
                if isinstance(extraction, basestring):
                    extraction = [extraction]

                for e_key in extraction:
                    value = extracted_dict.get(self.append_prefix(e_key))
                    if value:
                        result[self.append_prefix(var.name, True)] = value
                        result[var.name] = value
                        continue

                    value = extracted_dict.get(e_key)

                    if value:
                        result[var.name] = value

        self.update(**result)

    def _get_values(self):
        """ Return values for extraction type
        :rtype: dict
        """
        return self.kwargs

    def collect(self):
        self._extract()

    def append_prefix(self, key, is_module=False):
        if is_module:
            return MODULE_PREFIX + key
        return self.PREFIX + '_' + key

    def update(self, **kwargs):
        for k, v in kwargs.items():
            setattr(self, k, v)
            self._variables_dict[k] = v

    def update_by_instance(self, instance):
        self.update(**instance.to_dict())

    def to_dict(self):
        if not self.__collected:
            self.collect()
        return self._variables_dict


class DefaultsVariableBox(VariableBox):
    def __init__(self, variables_prefix=None, custom_variables=None, **kwargs):
        super(DefaultsVariableBox, self).__init__(variables_prefix,
                                                  custom_variables, **kwargs)
        # set default value for variable
        for var in self.all_variables:
            var_name = var.name
            default = var.default
            if custom_variables and var_name in custom_variables:
                default = custom_variables[var_name].default
            self.update(**{var_name: default})

    def __eq__(self, other):
        other_dict = other.to_dict()
        self_dict = self.to_dict()
        result = {k: other_dict[k] for k in self_dict if k in other_dict and
                  self_dict[k] != other_dict[k]}
        return not bool(result)

    def __ne__(self, other):
        return not self == other


class ArgsVariableBox(VariableBox):
    """ Extract variables from sys args represented as dict"""

    def _get_values(self):
        return {k: v for k, v in self.kwargs.items()
                if k in self.all_variables_names}


class SysEnvVariableBox(VariableBox):
    """ Extract variables from system ENVIRONMENT"""

    def __init__(self, variables_prefix=None, custom_variables=None, **kwargs):
        super(SysEnvVariableBox, self).__init__(variables_prefix,
                                                custom_variables, **kwargs)
        self.kwargs = kwargs or {}

    @staticmethod
    def _parse_value(value, is_collection=False):
        if is_collection:
            value = value.split(',')
        return value

    def _get_values(self):
        env_dict = {k: self.kwargs.get(k) for k in self.all_variables_names
                    if self.kwargs.get(k)}
        env_dict.update({k: self._parse_value(os.getenv(k),
                                              k in self.collection_variables)
                         for k in self.all_variables_names if os.getenv(k)})
        return env_dict


class FileVariableBox(VariableBox):
    """ Extract variables from file- script exports to sys ENVIRONMENT """

    def __init__(self, config_file='', **kwargs):
        super(FileVariableBox, self).__init__(**kwargs)
        self.file_path = config_file

    def _get_values(self):
        """ Parsing file of strings like """
        _variables = {}

        for line in self._get_lines():
            line = line.strip()
            if line and ' ' in line and '=' in line:
                _export, expression = line.split(' ')

                data = expression.split('=')
                if len(data) < 2 or not all(data):
                    continue

                k = data[0]
                v = '='.join(data[1:])
                if k in self.all_variables_names:
                    _variables[k] = v

        return _variables

    def _get_lines(self):
        # Valid line is line like "export OS_USERNAME=admin"
        lines = []
        if os.path.exists(self.file_path):
            with open(self.file_path) as file_handler:
                lines = file_handler.readlines()

        return lines


class YamlFileBox(FileVariableBox):
    """ Extract variables from yaml file"""

    def fill_vars_by_content(self, dict_obj, var_dict, matching_dict=None):
        matching_dict = matching_dict or get_yaml_variables_matching()
        msg = 'Invalid variable in yaml file: {0}'
        for key, value in dict_obj.items():
            if isinstance(value, dict):
                inner_dict = matching_dict.get(key)

                if inner_dict is None:
                    logger.debug(msg.format(key))
                    continue

                self.fill_vars_by_content(value, var_dict, inner_dict)
            else:
                variable_name = matching_dict.get(key)

                if variable_name is None:
                    logger.debug(msg.format(key))
                    continue

                if self.PREFIX:
                    var_dict[self.append_prefix(variable_name, True)] = value
                var_dict[variable_name] = value

    def _extract(self):
        """ Extract all variables in desired order"""
        self.__collected = True
        _variables = {}
        try:
            with open(self.file_path) as file_handler:
                content = yaml.load(file_handler)
        except EnvironmentError:
            pass
        else:
            self.fill_vars_by_content(content, _variables)
        self.update(**_variables)


def write_variables(filepath):
    # Write down all variables with extraction map
    from operator import attrgetter

    with open(filepath, 'w') as fh:
        variable_delimiter = '\n\n'
        for variable in sorted(variables, key=attrgetter('name')):
            data_list = [
                variable.name,
                "Variable default value: " + str(variable.default),
            ]
            extraction_str = "Variable extraction sequence: "
            if isinstance(variable.extraction, basestring):
                extraction_str += variable.extraction or variable.name
            else:
                extraction_str += ', '.join(variable.extraction or
                                            [variable.name])

            data_list.append(extraction_str)

            data_line = '\n'.join(data_list)
            data_line += variable_delimiter
            fh.write(data_line)


if __name__ == '__main__':
    write_variables("variables.txt")
